@@ -1,14 +1,21 @@
 from __future__ import annotations
 
 import abc
+from collections import defaultdict
+from functools import partial
 import inspect
+import re
 from typing import (
     TYPE_CHECKING,
     Any,
+    Callable,
+    DefaultDict,
     Dict,
     Hashable,
+    Iterable,
     Iterator,
     List,
+    Sequence,
     cast,
 )
 import warnings
@@ -24,10 +31,18 @@ from pandas._typing import (
     AggFuncTypeDict,
     AggObjType,
     Axis,
-    FrameOrSeries,
-    FrameOrSeriesUnion,
+    NDFrameT,
+    npt,
+)
+from pandas.errors import (
+    DataError,
+    SpecificationError,
 )
 from pandas.util._decorators import cache_readonly
+from pandas.util._exceptions import (
+    find_stack_level,
+    rewrite_warning,
+)
 
 from pandas.core.dtypes.cast import is_nested_object
 from pandas.core.dtypes.common import (
@@ -43,14 +58,9 @@ from pandas.core.dtypes.generic import (
 )
 
 from pandas.core.algorithms import safe_sort
-from pandas.core.base import (
-    DataError,
-    SelectionMixin,
-    SpecificationError,
-)
+from pandas.core.base import SelectionMixin
 import pandas.core.common as com
 from pandas.core.construction import (
-    array as pd_array,
     create_series_with_explicit_dtype,
     ensure_wrapped_if_datetimelike,
 )
@@ -65,6 +75,7 @@ if TYPE_CHECKING:
     from pandas.core.resample import Resampler
     from pandas.core.window.rolling import BaseWindow
 
+
 ResType = Dict[int, Any]
 
 
@@ -106,7 +117,7 @@ class Apply(metaclass=abc.ABCMeta):
         result_type: str | None,
         args,
         kwargs,
-    ):
+    ) -> None:
         self.obj = obj
         self.raw = raw
         self.args = args or ()
@@ -137,10 +148,10 @@ class Apply(metaclass=abc.ABCMeta):
         self.f: AggFuncType = f
 
     @abc.abstractmethod
-    def apply(self) -> FrameOrSeriesUnion:
+    def apply(self) -> DataFrame | Series:
         pass
 
-    def agg(self) -> FrameOrSeriesUnion | None:
+    def agg(self) -> DataFrame | Series | None:
         """
         Provide an implementation for the aggregators.
 
@@ -166,12 +177,20 @@ class Apply(metaclass=abc.ABCMeta):
         if callable(arg):
             f = com.get_cython_func(arg)
             if f and not args and not kwargs:
-                return getattr(obj, f)()
+                # GH#50538
+                old_msg = "The default value of numeric_only"
+                new_msg = (
+                    f"The operation {arg} failed on a column. If any error is "
+                    f"raised, this will raise an exception in a future version "
+                    f"of pandas. Drop these columns to avoid this warning."
+                )
+                with rewrite_warning(old_msg, FutureWarning, new_msg):
+                    return getattr(obj, f)()
 
         # caller can react
         return None
 
-    def transform(self) -> FrameOrSeriesUnion:
+    def transform(self) -> DataFrame | Series:
         """
         Transform a DataFrame or Series.
 
@@ -227,8 +246,12 @@ class Apply(metaclass=abc.ABCMeta):
             and not obj.empty
         ):
             raise ValueError("Transform function failed")
+        # error: Argument 1 to "__get__" of "AxisProperty" has incompatible type
+        # "Union[Series, DataFrame, GroupBy[Any], SeriesGroupBy,
+        # DataFrameGroupBy, BaseWindow, Resampler]"; expected "Union[DataFrame,
+        # Series]"
         if not isinstance(result, (ABCSeries, ABCDataFrame)) or not result.index.equals(
-            obj.index
+            obj.index  # type:ignore[arg-type]
         ):
             raise ValueError("Function did not transform")
 
@@ -252,7 +275,7 @@ class Apply(metaclass=abc.ABCMeta):
 
         func = self.normalize_dictlike_arg("transform", obj, func)
 
-        results: dict[Hashable, FrameOrSeriesUnion] = {}
+        results: dict[Hashable, DataFrame | Series] = {}
         failed_names = []
         all_type_errors = True
         for name, how in func.items():
@@ -265,8 +288,9 @@ class Apply(metaclass=abc.ABCMeta):
                     "No transform functions were provided",
                 }:
                     raise err
-                elif not isinstance(err, TypeError):
-                    all_type_errors = False
+                else:
+                    if not isinstance(err, TypeError):
+                        all_type_errors = False
                     failed_names.append(name)
         # combine results
         if not results:
@@ -274,16 +298,15 @@ class Apply(metaclass=abc.ABCMeta):
             raise klass("Transform function failed")
         if len(failed_names) > 0:
             warnings.warn(
-                f"{failed_names} did not transform successfully and did not raise "
-                f"a TypeError. If any error is raised except for TypeError, "
-                f"this will raise in a future version of pandas. "
+                f"{failed_names} did not transform successfully. If any error is "
+                f"raised, this will raise in a future version of pandas. "
                 f"Drop these columns/ops to avoid this warning.",
                 FutureWarning,
-                stacklevel=4,
+                stacklevel=find_stack_level(),
             )
         return concat(results, axis=1)
 
-    def transform_str_or_callable(self, func) -> FrameOrSeriesUnion:
+    def transform_str_or_callable(self, func) -> DataFrame | Series:
         """
         Compute transform in the case of a string or callable func
         """
@@ -297,7 +320,14 @@ class Apply(metaclass=abc.ABCMeta):
         if not args and not kwargs:
             f = com.get_cython_func(func)
             if f:
-                return getattr(obj, f)()
+                old_msg = "The default value of numeric_only"
+                new_msg = (
+                    f"The operation {func} failed on a column. If any error is "
+                    f"raised, this will raise an exception in a future version "
+                    f"of pandas. Drop these columns to avoid this warning."
+                )
+                with rewrite_warning(old_msg, FutureWarning, new_msg):
+                    return getattr(obj, f)()
 
         # Two possible ways to use a UDF - apply or call directly
         try:
@@ -305,7 +335,7 @@ class Apply(metaclass=abc.ABCMeta):
         except Exception:
             return func(obj, *args, **kwargs)
 
-    def agg_list_like(self) -> FrameOrSeriesUnion:
+    def agg_list_like(self) -> DataFrame | Series:
         """
         Compute aggregation in the case of a list-like argument.
 
@@ -318,6 +348,9 @@ class Apply(metaclass=abc.ABCMeta):
         obj = self.obj
         arg = cast(List[AggFuncTypeBase], self.f)
 
+        if getattr(obj, "axis", 0) == 1:
+            raise NotImplementedError("axis other than 0 is not supported")
+
         if not isinstance(obj, SelectionMixin):
             # i.e. obj is Series or DataFrame
             selected_obj = obj
@@ -329,6 +362,13 @@ class Apply(metaclass=abc.ABCMeta):
 
         results = []
         keys = []
+        failed_names = []
+
+        depr_nuisance_columns_msg = (
+            "{} did not aggregate successfully. If any error is "
+            "raised this will raise in a future version of pandas. "
+            "Drop these columns/ops to avoid this warning."
+        )
 
         # degenerate case
         if selected_obj.ndim == 1:
@@ -338,7 +378,7 @@ class Apply(metaclass=abc.ABCMeta):
                     new_res = colg.aggregate(a)
 
                 except TypeError:
-                    pass
+                    failed_names.append(com.get_callable_name(a) or a)
                 else:
                     results.append(new_res)
 
@@ -352,20 +392,39 @@ class Apply(metaclass=abc.ABCMeta):
             for index, col in enumerate(selected_obj):
                 colg = obj._gotitem(col, ndim=1, subset=selected_obj.iloc[:, index])
                 try:
-                    new_res = colg.aggregate(arg)
+                    # Capture and suppress any warnings emitted by us in the call
+                    # to agg below, but pass through any warnings that were
+                    # generated otherwise.
+                    # This is necessary because of https://bugs.python.org/issue29672
+                    # See GH #43741 for more details
+                    with warnings.catch_warnings(record=True) as record:
+                        new_res = colg.aggregate(arg)
+                    if len(record) > 0:
+                        match = re.compile(depr_nuisance_columns_msg.format(".*"))
+                        for warning in record:
+                            if re.match(match, str(warning.message)):
+                                failed_names.append(col)
+                            else:
+                                warnings.warn_explicit(
+                                    message=warning.message,
+                                    category=warning.category,
+                                    filename=warning.filename,
+                                    lineno=warning.lineno,
+                                )
+
                 except (TypeError, DataError):
-                    pass
+                    failed_names.append(col)
                 except ValueError as err:
                     # cannot aggregate
                     if "Must produce aggregated value" in str(err):
                         # raised directly in _aggregate_named
-                        pass
+                        failed_names.append(col)
                     elif "no results" in str(err):
                         # reached in test_frame_apply.test_nuiscance_columns
                         #  where the colg.aggregate(arg) ends up going through
                         #  the selected_obj.ndim == 1 branch above with arg == ["sum"]
                         #  on a datetime64[ns] column
-                        pass
+                        failed_names.append(col)
                     else:
                         raise
                 else:
@@ -378,6 +437,13 @@ class Apply(metaclass=abc.ABCMeta):
         if not len(results):
             raise ValueError("no results")
 
+        if len(failed_names) > 0:
+            warnings.warn(
+                depr_nuisance_columns_msg.format(failed_names),
+                FutureWarning,
+                stacklevel=find_stack_level(),
+            )
+
         try:
             concatenated = concat(results, keys=keys, axis=1, sort=False)
         except TypeError as err:
@@ -402,7 +468,7 @@ class Apply(metaclass=abc.ABCMeta):
             )
             return concatenated.reindex(full_ordered_index, copy=False)
 
-    def agg_dict_like(self) -> FrameOrSeriesUnion:
+    def agg_dict_like(self) -> DataFrame | Series:
         """
         Compute aggregation in the case of a dict-like argument.
 
@@ -416,6 +482,9 @@ class Apply(metaclass=abc.ABCMeta):
         obj = self.obj
         arg = cast(AggFuncTypeDict, self.f)
 
+        if getattr(obj, "axis", 0) == 1:
+            raise NotImplementedError("axis other than 0 is not supported")
+
         if not isinstance(obj, SelectionMixin):
             # i.e. obj is Series or DataFrame
             selected_obj = obj
@@ -444,6 +513,7 @@ class Apply(metaclass=abc.ABCMeta):
 
         # combine results
         if all(is_ndframe):
+            keys_to_use: Iterable[Hashable]
             keys_to_use = [k for k in keys if not results[k].empty]
             # Have to check, if at least one DataFrame is not empty.
             keys_to_use = keys_to_use if keys_to_use != [] else keys
@@ -451,9 +521,7 @@ class Apply(metaclass=abc.ABCMeta):
                 # keys are columns, so we can preserve names
                 ktu = Index(keys_to_use)
                 ktu._set_names(selected_obj.columns.names)
-                # Incompatible types in assignment (expression has type "Index",
-                # variable has type "List[Hashable]")
-                keys_to_use = ktu  # type: ignore[assignment]
+                keys_to_use = ktu
 
             axis = 0 if isinstance(obj, ABCSeries) else 1
             result = concat(
@@ -481,7 +549,7 @@ class Apply(metaclass=abc.ABCMeta):
 
         return result
 
-    def apply_str(self) -> FrameOrSeriesUnion:
+    def apply_str(self) -> DataFrame | Series:
         """
         Compute apply in case of a string.
 
@@ -500,13 +568,18 @@ class Apply(metaclass=abc.ABCMeta):
         func = getattr(obj, f, None)
         if callable(func):
             sig = inspect.getfullargspec(func)
-            if "axis" in sig.args:
+            arg_names = (*sig.args, *sig.kwonlyargs)
+            if self.axis != 0 and (
+                "axis" not in arg_names or f in ("corrwith", "mad", "skew")
+            ):
+                raise ValueError(f"Operation {f} does not support axis=1")
+            elif "axis" in arg_names:
                 self.kwargs["axis"] = self.axis
             elif self.axis != 0:
                 raise ValueError(f"Operation {f} does not support axis=1")
         return self._try_aggregate_string_function(obj, f, *self.args, **self.kwargs)
 
-    def apply_multiple(self) -> FrameOrSeriesUnion:
+    def apply_multiple(self) -> DataFrame | Series:
         """
         Compute apply in case of a list-like or dict-like.
 
@@ -518,7 +591,7 @@ class Apply(metaclass=abc.ABCMeta):
         return self.obj.aggregate(self.f, self.axis, *self.args, **self.kwargs)
 
     def normalize_dictlike_arg(
-        self, how: str, obj: FrameOrSeriesUnion, func: AggFuncTypeDict
+        self, how: str, obj: DataFrame | Series, func: AggFuncTypeDict
     ) -> AggFuncTypeDict:
         """
         Handler for dict-like argument.
@@ -545,18 +618,17 @@ class Apply(metaclass=abc.ABCMeta):
                 cols_sorted = list(safe_sort(list(cols)))
                 raise KeyError(f"Column(s) {cols_sorted} do not exist")
 
-        is_aggregator = lambda x: isinstance(x, (list, tuple, dict))
+        aggregator_types = (list, tuple, dict)
 
         # if we have a dict of any non-scalars
         # eg. {'A' : ['mean']}, normalize all to
         # be list-likes
         # Cannot use func.values() because arg may be a Series
-        if any(is_aggregator(x) for _, x in func.items()):
+        if any(isinstance(x, aggregator_types) for _, x in func.items()):
             new_func: AggFuncTypeDict = {}
             for k, v in func.items():
-                if not is_aggregator(v):
-                    # mypy can't realize v is not a list here
-                    new_func[k] = [v]  # type:ignore[list-item]
+                if not isinstance(v, aggregator_types):
+                    new_func[k] = [v]
                 else:
                     new_func[k] = v
             func = new_func
@@ -600,7 +672,11 @@ class NDFrameApply(Apply):
 
     @property
     def index(self) -> Index:
-        return self.obj.index
+        # error: Argument 1 to "__get__" of "AxisProperty" has incompatible type
+        # "Union[Series, DataFrame, GroupBy[Any], SeriesGroupBy,
+        # DataFrameGroupBy, BaseWindow, Resampler]"; expected "Union[DataFrame,
+        # Series]"
+        return self.obj.index  # type:ignore[arg-type]
 
     @property
     def agg_axis(self) -> Index:
@@ -631,7 +707,7 @@ class FrameApply(NDFrameApply):
     @abc.abstractmethod
     def wrap_results_for_axis(
         self, results: ResType, res_index: Index
-    ) -> FrameOrSeriesUnion:
+    ) -> DataFrame | Series:
         pass
 
     # ---------------------------------------------------------------
@@ -652,7 +728,7 @@ class FrameApply(NDFrameApply):
     def dtypes(self) -> Series:
         return self.obj.dtypes
 
-    def apply(self) -> FrameOrSeriesUnion:
+    def apply(self) -> DataFrame | Series:
         """compute the results"""
         # dispatch to agg
         if is_list_like(self.f):
@@ -737,7 +813,10 @@ class FrameApply(NDFrameApply):
 
         if not should_reduce:
             try:
-                r = self.f(Series([], dtype=np.float64))
+                if self.axis == 0:
+                    r = self.f(Series([], dtype=np.float64))
+                else:
+                    r = self.f(Series(index=self.columns, dtype=np.float64))
             except Exception:
                 pass
             else:
@@ -833,7 +912,7 @@ class FrameApply(NDFrameApply):
 
         return results, res_index
 
-    def wrap_results(self, results: ResType, res_index: Index) -> FrameOrSeriesUnion:
+    def wrap_results(self, results: ResType, res_index: Index) -> DataFrame | Series:
         from pandas import Series
 
         # see if we can infer the results
@@ -856,7 +935,7 @@ class FrameApply(NDFrameApply):
 
         return result
 
-    def apply_str(self) -> FrameOrSeriesUnion:
+    def apply_str(self) -> DataFrame | Series:
         # Caller is responsible for checking isinstance(self.f, str)
         # TODO: GH#39993 - Avoid special-casing by replacing with lambda
         if self.f == "size":
@@ -887,7 +966,7 @@ class FrameRowApply(FrameApply):
 
     def wrap_results_for_axis(
         self, results: ResType, res_index: Index
-    ) -> FrameOrSeriesUnion:
+    ) -> DataFrame | Series:
         """return the results for the rows"""
 
         if self.result_type == "reduce":
@@ -957,7 +1036,7 @@ class FrameColumnApply(FrameApply):
                 # GH#35462 re-pin mgr in case setitem changed it
                 ser._mgr = mgr
                 mgr.set_values(arr)
-                ser.name = name
+                object.__setattr__(ser, "_name", name)
                 yield ser
 
     @property
@@ -970,9 +1049,9 @@ class FrameColumnApply(FrameApply):
 
     def wrap_results_for_axis(
         self, results: ResType, res_index: Index
-    ) -> FrameOrSeriesUnion:
+    ) -> DataFrame | Series:
         """return the results for the columns"""
-        result: FrameOrSeriesUnion
+        result: DataFrame | Series
 
         # we have requested to expand
         if self.result_type == "expand":
@@ -1014,7 +1093,7 @@ class SeriesApply(NDFrameApply):
         convert_dtype: bool,
         args,
         kwargs,
-    ):
+    ) -> None:
         self.convert_dtype = convert_dtype
 
         super().__init__(
@@ -1026,7 +1105,7 @@ class SeriesApply(NDFrameApply):
             kwargs=kwargs,
         )
 
-    def apply(self) -> FrameOrSeriesUnion:
+    def apply(self) -> DataFrame | Series:
         obj = self.obj
 
         if len(obj) == 0:
@@ -1040,6 +1119,7 @@ class SeriesApply(NDFrameApply):
             # if we are a string, try to dispatch
             return self.apply_str()
 
+        # self.f is Callable
         return self.apply_standard()
 
     def agg(self):
@@ -1076,8 +1156,9 @@ class SeriesApply(NDFrameApply):
             obj, method="apply"
         )
 
-    def apply_standard(self) -> FrameOrSeriesUnion:
-        f = self.f
+    def apply_standard(self) -> DataFrame | Series:
+        # caller is responsible for ensuring that f is Callable
+        f = cast(Callable, self.f)
         obj = self.obj
 
         with np.errstate(all="ignore"):
@@ -1090,21 +1171,16 @@ class SeriesApply(NDFrameApply):
                 mapped = obj._values.map(f)
             else:
                 values = obj.astype(object)._values
-                # error: Argument 2 to "map_infer" has incompatible type
-                # "Union[Callable[..., Any], str, List[Union[Callable[..., Any], str]],
-                # Dict[Hashable, Union[Union[Callable[..., Any], str],
-                # List[Union[Callable[..., Any], str]]]]]"; expected
-                # "Callable[[Any], Any]"
                 mapped = lib.map_infer(
                     values,
-                    f,  # type: ignore[arg-type]
+                    f,
                     convert=self.convert_dtype,
                 )
 
         if len(mapped) and isinstance(mapped[0], ABCSeries):
-            # GH 25959 use pd.array instead of tolist
-            # so extension arrays can be used
-            return obj._constructor_expanddim(pd_array(mapped), index=obj.index)
+            # GH#43986 Need to do list(mapped) in order to get treated as nested
+            #  See also GH#25959 regarding EA support
+            return obj._constructor_expanddim(list(mapped), index=obj.index)
         else:
             return obj._constructor(mapped, index=obj.index).__finalize__(
                 obj, method="apply"
@@ -1114,11 +1190,11 @@ class SeriesApply(NDFrameApply):
 class GroupByApply(Apply):
     def __init__(
         self,
-        obj: GroupBy[FrameOrSeries],
+        obj: GroupBy[NDFrameT],
         func: AggFuncType,
         args,
         kwargs,
-    ):
+    ) -> None:
         kwargs = kwargs.copy()
         self.axis = obj.obj._get_axis_number(kwargs.get("axis", 0))
         super().__init__(
@@ -1147,7 +1223,7 @@ class ResamplerWindowApply(Apply):
         func: AggFuncType,
         args,
         kwargs,
-    ):
+    ) -> None:
         super().__init__(
             obj,
             func,
@@ -1162,3 +1238,366 @@ class ResamplerWindowApply(Apply):
 
     def transform(self):
         raise NotImplementedError
+
+
+def reconstruct_func(
+    func: AggFuncType | None, **kwargs
+) -> tuple[bool, AggFuncType | None, list[str] | None, npt.NDArray[np.intp] | None]:
+    """
+    This is the internal function to reconstruct func given if there is relabeling
+    or not and also normalize the keyword to get new order of columns.
+
+    If named aggregation is applied, `func` will be None, and kwargs contains the
+    column and aggregation function information to be parsed;
+    If named aggregation is not applied, `func` is either string (e.g. 'min') or
+    Callable, or list of them (e.g. ['min', np.max]), or the dictionary of column name
+    and str/Callable/list of them (e.g. {'A': 'min'}, or {'A': [np.min, lambda x: x]})
+
+    If relabeling is True, will return relabeling, reconstructed func, column
+    names, and the reconstructed order of columns.
+    If relabeling is False, the columns and order will be None.
+
+    Parameters
+    ----------
+    func: agg function (e.g. 'min' or Callable) or list of agg functions
+        (e.g. ['min', np.max]) or dictionary (e.g. {'A': ['min', np.max]}).
+    **kwargs: dict, kwargs used in is_multi_agg_with_relabel and
+        normalize_keyword_aggregation function for relabelling
+
+    Returns
+    -------
+    relabelling: bool, if there is relabelling or not
+    func: normalized and mangled func
+    columns: list of column names
+    order: array of columns indices
+
+    Examples
+    --------
+    >>> reconstruct_func(None, **{"foo": ("col", "min")})
+    (True, defaultdict(<class 'list'>, {'col': ['min']}), ('foo',), array([0]))
+
+    >>> reconstruct_func("min")
+    (False, 'min', None, None)
+    """
+    relabeling = func is None and is_multi_agg_with_relabel(**kwargs)
+    columns: list[str] | None = None
+    order: npt.NDArray[np.intp] | None = None
+
+    if not relabeling:
+        if isinstance(func, list) and len(func) > len(set(func)):
+
+            # GH 28426 will raise error if duplicated function names are used and
+            # there is no reassigned name
+            raise SpecificationError(
+                "Function names must be unique if there is no new column names "
+                "assigned"
+            )
+        elif func is None:
+            # nicer error message
+            raise TypeError("Must provide 'func' or tuples of '(column, aggfunc).")
+
+    if relabeling:
+        func, columns, order = normalize_keyword_aggregation(kwargs)
+
+    return relabeling, func, columns, order
+
+
+def is_multi_agg_with_relabel(**kwargs) -> bool:
+    """
+    Check whether kwargs passed to .agg look like multi-agg with relabeling.
+
+    Parameters
+    ----------
+    **kwargs : dict
+
+    Returns
+    -------
+    bool
+
+    Examples
+    --------
+    >>> is_multi_agg_with_relabel(a="max")
+    False
+    >>> is_multi_agg_with_relabel(a_max=("a", "max"), a_min=("a", "min"))
+    True
+    >>> is_multi_agg_with_relabel()
+    False
+    """
+    return all(isinstance(v, tuple) and len(v) == 2 for v in kwargs.values()) and (
+        len(kwargs) > 0
+    )
+
+
+def normalize_keyword_aggregation(
+    kwargs: dict,
+) -> tuple[dict, list[str], npt.NDArray[np.intp]]:
+    """
+    Normalize user-provided "named aggregation" kwargs.
+    Transforms from the new ``Mapping[str, NamedAgg]`` style kwargs
+    to the old Dict[str, List[scalar]]].
+
+    Parameters
+    ----------
+    kwargs : dict
+
+    Returns
+    -------
+    aggspec : dict
+        The transformed kwargs.
+    columns : List[str]
+        The user-provided keys.
+    col_idx_order : List[int]
+        List of columns indices.
+
+    Examples
+    --------
+    >>> normalize_keyword_aggregation({"output": ("input", "sum")})
+    (defaultdict(<class 'list'>, {'input': ['sum']}), ('output',), array([0]))
+    """
+    from pandas.core.indexes.base import Index
+
+    # Normalize the aggregation functions as Mapping[column, List[func]],
+    # process normally, then fixup the names.
+    # TODO: aggspec type: typing.Dict[str, List[AggScalar]]
+    # May be hitting https://github.com/python/mypy/issues/5958
+    # saying it doesn't have an attribute __name__
+    aggspec: DefaultDict = defaultdict(list)
+    order = []
+    columns, pairs = list(zip(*kwargs.items()))
+
+    for column, aggfunc in pairs:
+        aggspec[column].append(aggfunc)
+        order.append((column, com.get_callable_name(aggfunc) or aggfunc))
+
+    # uniquify aggfunc name if duplicated in order list
+    uniquified_order = _make_unique_kwarg_list(order)
+
+    # GH 25719, due to aggspec will change the order of assigned columns in aggregation
+    # uniquified_aggspec will store uniquified order list and will compare it with order
+    # based on index
+    aggspec_order = [
+        (column, com.get_callable_name(aggfunc) or aggfunc)
+        for column, aggfuncs in aggspec.items()
+        for aggfunc in aggfuncs
+    ]
+    uniquified_aggspec = _make_unique_kwarg_list(aggspec_order)
+
+    # get the new index of columns by comparison
+    col_idx_order = Index(uniquified_aggspec).get_indexer(uniquified_order)
+    return aggspec, columns, col_idx_order
+
+
+def _make_unique_kwarg_list(
+    seq: Sequence[tuple[Any, Any]]
+) -> Sequence[tuple[Any, Any]]:
+    """
+    Uniquify aggfunc name of the pairs in the order list
+
+    Examples:
+    --------
+    >>> kwarg_list = [('a', '<lambda>'), ('a', '<lambda>'), ('b', '<lambda>')]
+    >>> _make_unique_kwarg_list(kwarg_list)
+    [('a', '<lambda>_0'), ('a', '<lambda>_1'), ('b', '<lambda>')]
+    """
+    return [
+        (pair[0], "_".join([pair[1], str(seq[:i].count(pair))]))
+        if seq.count(pair) > 1
+        else pair
+        for i, pair in enumerate(seq)
+    ]
+
+
+def relabel_result(
+    result: DataFrame | Series,
+    func: dict[str, list[Callable | str]],
+    columns: Iterable[Hashable],
+    order: Iterable[int],
+) -> dict[Hashable, Series]:
+    """
+    Internal function to reorder result if relabelling is True for
+    dataframe.agg, and return the reordered result in dict.
+
+    Parameters:
+    ----------
+    result: Result from aggregation
+    func: Dict of (column name, funcs)
+    columns: New columns name for relabelling
+    order: New order for relabelling
+
+    Examples:
+    ---------
+    >>> result = DataFrame({"A": [np.nan, 2, np.nan],
+    ...       "C": [6, np.nan, np.nan], "B": [np.nan, 4, 2.5]})  # doctest: +SKIP
+    >>> funcs = {"A": ["max"], "C": ["max"], "B": ["mean", "min"]}
+    >>> columns = ("foo", "aab", "bar", "dat")
+    >>> order = [0, 1, 2, 3]
+    >>> _relabel_result(result, func, columns, order)  # doctest: +SKIP
+    dict(A=Series([2.0, NaN, NaN, NaN], index=["foo", "aab", "bar", "dat"]),
+         C=Series([NaN, 6.0, NaN, NaN], index=["foo", "aab", "bar", "dat"]),
+         B=Series([NaN, NaN, 2.5, 4.0], index=["foo", "aab", "bar", "dat"]))
+    """
+    from pandas.core.indexes.base import Index
+
+    reordered_indexes = [
+        pair[0] for pair in sorted(zip(columns, order), key=lambda t: t[1])
+    ]
+    reordered_result_in_dict: dict[Hashable, Series] = {}
+    idx = 0
+
+    reorder_mask = not isinstance(result, ABCSeries) and len(result.columns) > 1
+    for col, fun in func.items():
+        s = result[col].dropna()
+
+        # In the `_aggregate`, the callable names are obtained and used in `result`, and
+        # these names are ordered alphabetically. e.g.
+        #           C2   C1
+        # <lambda>   1  NaN
+        # amax     NaN  4.0
+        # max      NaN  4.0
+        # sum     18.0  6.0
+        # Therefore, the order of functions for each column could be shuffled
+        # accordingly so need to get the callable name if it is not parsed names, and
+        # reorder the aggregated result for each column.
+        # e.g. if df.agg(c1=("C2", sum), c2=("C2", lambda x: min(x))), correct order is
+        # [sum, <lambda>], but in `result`, it will be [<lambda>, sum], and we need to
+        # reorder so that aggregated values map to their functions regarding the order.
+
+        # However there is only one column being used for aggregation, not need to
+        # reorder since the index is not sorted, and keep as is in `funcs`, e.g.
+        #         A
+        # min   1.0
+        # mean  1.5
+        # mean  1.5
+        if reorder_mask:
+            fun = [
+                com.get_callable_name(f) if not isinstance(f, str) else f for f in fun
+            ]
+            col_idx_order = Index(s.index).get_indexer(fun)
+            s = s[col_idx_order]
+
+        # assign the new user-provided "named aggregation" as index names, and reindex
+        # it based on the whole user-provided names.
+        s.index = reordered_indexes[idx : idx + len(fun)]
+        reordered_result_in_dict[col] = s.reindex(columns, copy=False)
+        idx = idx + len(fun)
+    return reordered_result_in_dict
+
+
+# TODO: Can't use, because mypy doesn't like us setting __name__
+#   error: "partial[Any]" has no attribute "__name__"
+# the type is:
+#   typing.Sequence[Callable[..., ScalarResult]]
+#     -> typing.Sequence[Callable[..., ScalarResult]]:
+
+
+def _managle_lambda_list(aggfuncs: Sequence[Any]) -> Sequence[Any]:
+    """
+    Possibly mangle a list of aggfuncs.
+
+    Parameters
+    ----------
+    aggfuncs : Sequence
+
+    Returns
+    -------
+    mangled: list-like
+        A new AggSpec sequence, where lambdas have been converted
+        to have unique names.
+
+    Notes
+    -----
+    If just one aggfunc is passed, the name will not be mangled.
+    """
+    if len(aggfuncs) <= 1:
+        # don't mangle for .agg([lambda x: .])
+        return aggfuncs
+    i = 0
+    mangled_aggfuncs = []
+    for aggfunc in aggfuncs:
+        if com.get_callable_name(aggfunc) == "<lambda>":
+            aggfunc = partial(aggfunc)
+            aggfunc.__name__ = f"<lambda_{i}>"
+            i += 1
+        mangled_aggfuncs.append(aggfunc)
+
+    return mangled_aggfuncs
+
+
+def maybe_mangle_lambdas(agg_spec: Any) -> Any:
+    """
+    Make new lambdas with unique names.
+
+    Parameters
+    ----------
+    agg_spec : Any
+        An argument to GroupBy.agg.
+        Non-dict-like `agg_spec` are pass through as is.
+        For dict-like `agg_spec` a new spec is returned
+        with name-mangled lambdas.
+
+    Returns
+    -------
+    mangled : Any
+        Same type as the input.
+
+    Examples
+    --------
+    >>> maybe_mangle_lambdas('sum')
+    'sum'
+    >>> maybe_mangle_lambdas([lambda: 1, lambda: 2])  # doctest: +SKIP
+    [<function __main__.<lambda_0>,
+     <function pandas...._make_lambda.<locals>.f(*args, **kwargs)>]
+    """
+    is_dict = is_dict_like(agg_spec)
+    if not (is_dict or is_list_like(agg_spec)):
+        return agg_spec
+    mangled_aggspec = type(agg_spec)()  # dict or OrderedDict
+
+    if is_dict:
+        for key, aggfuncs in agg_spec.items():
+            if is_list_like(aggfuncs) and not is_dict_like(aggfuncs):
+                mangled_aggfuncs = _managle_lambda_list(aggfuncs)
+            else:
+                mangled_aggfuncs = aggfuncs
+
+            mangled_aggspec[key] = mangled_aggfuncs
+    else:
+        mangled_aggspec = _managle_lambda_list(agg_spec)
+
+    return mangled_aggspec
+
+
+def validate_func_kwargs(
+    kwargs: dict,
+) -> tuple[list[str], list[str | Callable[..., Any]]]:
+    """
+    Validates types of user-provided "named aggregation" kwargs.
+    `TypeError` is raised if aggfunc is not `str` or callable.
+
+    Parameters
+    ----------
+    kwargs : dict
+
+    Returns
+    -------
+    columns : List[str]
+        List of user-provied keys.
+    func : List[Union[str, callable[...,Any]]]
+        List of user-provided aggfuncs
+
+    Examples
+    --------
+    >>> validate_func_kwargs({'one': 'min', 'two': 'max'})
+    (['one', 'two'], ['min', 'max'])
+    """
+    tuple_given_message = "func is expected but received {} in **kwargs."
+    columns = list(kwargs)
+    func = []
+    for col_func in kwargs.values():
+        if not (isinstance(col_func, str) or callable(col_func)):
+            raise TypeError(tuple_given_message.format(type(col_func).__name__))
+        func.append(col_func)
+    if not columns:
+        no_arg_message = "Must provide 'func' or named aggregation **kwargs."
+        raise TypeError(no_arg_message)
+    return columns, func
