@@ -9,6 +9,8 @@ If you need to make sure options are available even before a certain
 module is imported, register them here rather than in the module.
 
 """
+from __future__ import annotations
+
 import os
 from typing import Callable
 import warnings
@@ -37,7 +39,7 @@ use_bottleneck_doc = """
 """
 
 
-def use_bottleneck_cb(key):
+def use_bottleneck_cb(key) -> None:
     from pandas.core import nanops
 
     nanops.set_use_bottleneck(cf.get_option(key))
@@ -51,7 +53,7 @@ use_numexpr_doc = """
 """
 
 
-def use_numexpr_cb(key):
+def use_numexpr_cb(key) -> None:
     from pandas.core.computation import expressions
 
     expressions.set_use_numexpr(cf.get_option(key))
@@ -65,7 +67,7 @@ use_numba_doc = """
 """
 
 
-def use_numba_cb(key):
+def use_numba_cb(key) -> None:
     from pandas.core.util import numba_
 
     numba_.set_use_numba(cf.get_option(key))
@@ -259,7 +261,7 @@ pc_width_doc = """
 
 pc_chop_threshold_doc = """
 : float or None
-    if set to a float value, all float values smaller then the given threshold
+    if set to a float value, all float values smaller than the given threshold
     will be displayed as exactly 0 by repr and friends.
 """
 
@@ -329,7 +331,7 @@ pc_latex_multirow = """
 """
 
 
-def table_schema_cb(key):
+def table_schema_cb(key) -> None:
     from pandas.io.formats.printing import enable_data_resource_formatter
 
     enable_data_resource_formatter(cf.get_option(key))
@@ -361,7 +363,17 @@ with cf.config_prefix("display"):
         float_format_doc,
         validator=is_one_of_factory([None, is_callable]),
     )
-    cf.register_option("column_space", 12, validator=is_int)
+
+    def _deprecate_column_space(key):
+        warnings.warn(
+            "column_space is deprecated and will be removed "
+            "in a future version. Use df.to_string(col_space=...) "
+            "instead.",
+            FutureWarning,
+            stacklevel=find_stack_level(),
+        )
+
+    cf.register_option("column_space", 12, validator=is_int, cb=_deprecate_column_space)
     cf.register_option(
         "max_info_rows",
         1690785,
@@ -490,7 +502,7 @@ use_inf_as_na_doc = """
 # or we'll hit circular deps.
 
 
-def use_inf_as_na_cb(key):
+def use_inf_as_na_cb(key) -> None:
     from pandas.core.dtypes.missing import _use_inf_as_na
 
     _use_inf_as_na(key)
@@ -527,6 +539,26 @@ with cf.config_prefix("mode"):
     )
 
 
+# TODO better name?
+copy_on_write_doc = """
+: bool
+    Use new copy-view behaviour using Copy-on-Write. Defaults to False,
+    unless overridden by the 'PANDAS_COPY_ON_WRITE' environment variable
+    (if set to "1" for True, needs to be set before pandas is imported).
+"""
+
+
+with cf.config_prefix("mode"):
+    cf.register_option(
+        "copy_on_write",
+        # Get the default from an environment variable, if set, otherwise defaults
+        # to False. This environment variable can be set for testing.
+        os.environ.get("PANDAS_COPY_ON_WRITE", "0") == "1",
+        copy_on_write_doc,
+        validator=is_bool,
+    )
+
+
 # user warnings
 chained_assignment = """
 : string
@@ -710,7 +742,7 @@ plotting_backend_doc = """
 """
 
 
-def register_plotting_backend_cb(key):
+def register_plotting_backend_cb(key) -> None:
     if key == "matplotlib":
         # We defer matplotlib validation, since it's the default
         return
@@ -736,7 +768,7 @@ register_converter_doc = """
 """
 
 
-def register_converter_cb(key):
+def register_converter_cb(key) -> None:
     from pandas.plotting import (
         deregister_matplotlib_converters,
         register_matplotlib_converters,
