@@ -18,7 +18,10 @@ import pytest
 
 from pandas.core.dtypes.dtypes import IntervalDtype
 
-from pandas import Interval
+from pandas import (
+    Interval,
+    Series,
+)
 from pandas.core.arrays import IntervalArray
 from pandas.tests.extension import base
 
@@ -90,55 +93,72 @@ class TestGetitem(BaseInterval, base.BaseGetitemTests):
     pass
 
 
+class TestIndex(base.BaseIndexTests):
+    pass
+
+
 class TestGrouping(BaseInterval, base.BaseGroupbyTests):
     pass
 
 
 class TestInterface(BaseInterval, base.BaseInterfaceTests):
-    def test_view(self, data):
-        # __setitem__ incorrectly makes a copy (GH#27147), so we only
-        #  have a smoke-test
-        data.view()
-
-
-class TestReduce(base.BaseNoReduceTests):
     pass
 
 
-class TestMethods(BaseInterval, base.BaseMethodsTests):
-    @pytest.mark.skip(reason="addition is not defined for intervals")
-    def test_combine_add(self, data_repeated):
-        pass
+class TestReduce(base.BaseNoReduceTests):
+    @pytest.mark.parametrize("skipna", [True, False])
+    def test_reduce_series_numeric(self, data, all_numeric_reductions, skipna):
+        op_name = all_numeric_reductions
+        ser = Series(data)
 
-    @pytest.mark.skip(reason="Not Applicable")
+        if op_name in ["min", "max"]:
+            # IntervalArray *does* implement these
+            assert getattr(ser, op_name)(skipna=skipna) in data
+            assert getattr(data, op_name)(skipna=skipna) in data
+            return
+
+        super().test_reduce_series_numeric(data, all_numeric_reductions, skipna)
+
+
+class TestMethods(BaseInterval, base.BaseMethodsTests):
+    @pytest.mark.xfail(reason="addition is not defined for intervals")
+    def test_combine_add(self, data_repeated):
+        super().test_combine_add(data_repeated)
+
+    @pytest.mark.xfail(
+        reason="Raises with incorrect message bc it disallows *all* listlikes "
+        "instead of just wrong-length listlikes"
+    )
     def test_fillna_length_mismatch(self, data_missing):
-        pass
+        super().test_fillna_length_mismatch(data_missing)
 
 
 class TestMissing(BaseInterval, base.BaseMissingTests):
-    # Index.fillna only accepts scalar `value`, so we have to skip all
+    # Index.fillna only accepts scalar `value`, so we have to xfail all
     # non-scalar fill tests.
-    unsupported_fill = pytest.mark.skip("Unsupported fillna option.")
+    unsupported_fill = pytest.mark.xfail(
+        reason="Unsupported fillna option for Interval."
+    )
 
     @unsupported_fill
     def test_fillna_limit_pad(self):
-        pass
+        super().test_fillna_limit_pad()
 
     @unsupported_fill
     def test_fillna_series_method(self):
-        pass
+        super().test_fillna_series_method()
 
     @unsupported_fill
     def test_fillna_limit_backfill(self):
-        pass
+        super().test_fillna_limit_backfill()
 
     @unsupported_fill
     def test_fillna_no_op_returns_copy(self):
-        pass
+        super().test_fillna_no_op_returns_copy()
 
     @unsupported_fill
     def test_fillna_series(self):
-        pass
+        super().test_fillna_series()
 
     def test_fillna_non_scalar_raises(self, data_missing):
         msg = "can only insert Interval objects and NA into an IntervalArray"
@@ -155,9 +175,9 @@ class TestSetitem(BaseInterval, base.BaseSetitemTests):
 
 
 class TestPrinting(BaseInterval, base.BasePrintingTests):
-    @pytest.mark.skip(reason="custom repr")
+    @pytest.mark.xfail(reason="Interval has custom repr")
     def test_array_repr(self, data, size):
-        pass
+        super().test_array_repr()
 
 
 class TestParsing(BaseInterval, base.BaseParsingTests):
