@@ -1,6 +1,12 @@
 import pytest
 
-from pandas import DatetimeIndex, Index, MultiIndex, Period, date_range
+from pandas import (
+    DatetimeIndex,
+    Index,
+    MultiIndex,
+    Period,
+    date_range,
+)
 import pandas._testing as tm
 
 
