@@ -5,7 +5,6 @@ from pandas._libs.tslibs.period import IncompatibleFrequency
 
 from pandas.core.dtypes.dtypes import PeriodDtype
 
-import pandas as pd
 from pandas import (
     Index,
     NaT,
@@ -184,9 +183,10 @@ class TestPeriodIndex:
         vals = np.arange(100000, 100000 + 10000, 100, dtype=np.int64)
         vals = vals.view(np.dtype("M8[us]"))
 
-        msg = r"Wrong dtype: datetime64\[us\]"
-        with pytest.raises(ValueError, match=msg):
-            PeriodIndex(vals, freq="D")
+        pi = PeriodIndex(vals, freq="D")
+
+        expected = PeriodIndex(vals.astype("M8[ns]"), freq="D")
+        tm.assert_index_equal(pi, expected)
 
     @pytest.mark.parametrize("box", [None, "series", "index"])
     def test_constructor_datetime64arr_ok(self, box):
@@ -195,7 +195,7 @@ class TestPeriodIndex:
         if box is None:
             data = data._values
         elif box == "series":
-            data = pd.Series(data)
+            data = Series(data)
 
         result = PeriodIndex(data, freq="D")
         expected = PeriodIndex(
@@ -330,7 +330,7 @@ class TestPeriodIndex:
         msg = "Should be numpy array of type i8"
         with pytest.raises(AssertionError, match=msg):
             # Need ndarray, not Int64Index
-            type(idx._data)._simple_new(idx.astype("i8"), freq=idx.freq)
+            type(idx._data)._simple_new(Index(idx.asi8), freq=idx.freq)
 
         arr = type(idx._data)._simple_new(idx.asi8, freq=idx.freq)
         result = idx._simple_new(arr, name="p")
@@ -362,7 +362,7 @@ class TestPeriodIndex:
             period_range(start="2011-01-01", end="NaT", freq="M")
 
     def test_constructor_year_and_quarter(self):
-        year = pd.Series([2001, 2002, 2003])
+        year = Series([2001, 2002, 2003])
         quarter = year - 2000
         idx = PeriodIndex(year=year, quarter=quarter)
         strs = [f"{t[0]:d}Q{t[1]:d}" for t in zip(quarter, year)]
@@ -513,12 +513,30 @@ class TestPeriodIndex:
         tm.assert_index_equal(res, expected)
 
 
-class TestSeriesPeriod:
-    def setup_method(self, method):
-        self.series = Series(period_range("2000-01-01", periods=10, freq="D"))
+class TestShallowCopy:
+    def test_shallow_copy_empty(self):
+        # GH#13067
+        idx = PeriodIndex([], freq="M")
+        result = idx._view()
+        expected = idx
 
+        tm.assert_index_equal(result, expected)
+
+    def test_shallow_copy_disallow_i8(self):
+        # GH#24391
+        pi = period_range("2018-01-01", periods=3, freq="2D")
+        with pytest.raises(AssertionError, match="ndarray"):
+            pi._shallow_copy(pi.asi8)
+
+    def test_shallow_copy_requires_disallow_period_index(self):
+        pi = period_range("2018-01-01", periods=3, freq="2D")
+        with pytest.raises(AssertionError, match="PeriodIndex"):
+            pi._shallow_copy(pi)
+
+
+class TestSeriesPeriod:
     def test_constructor_cant_cast_period(self):
-        msg = "Cannot cast PeriodArray to dtype float64"
+        msg = "Cannot cast PeriodIndex to dtype float64"
         with pytest.raises(TypeError, match=msg):
             Series(period_range("2000-01-01", periods=10, freq="D"), dtype=float)
 
