@@ -1,6 +1,7 @@
 import numpy as np
 import pytest
 
+from pandas.errors import SettingWithCopyError
 import pandas.util._test_decorators as td
 
 import pandas as pd
@@ -14,7 +15,6 @@ from pandas import (
     notna,
 )
 import pandas._testing as tm
-import pandas.core.common as com
 
 
 def assert_equal(a, b):
@@ -196,7 +196,7 @@ class TestMultiIndexSetItem:
         df.loc[4, "d"] = arr
         tm.assert_series_equal(df.loc[4, "d"], Series(arr, index=[8, 10], name="d"))
 
-    def test_multiindex_assignment_single_dtype(self, using_array_manager):
+    def test_multiindex_assignment_single_dtype(self, using_copy_on_write):
         # GH3777 part 2b
         # single dtype
         arr = np.array([0.0, 1.0])
@@ -214,10 +214,9 @@ class TestMultiIndexSetItem:
         exp = Series(arr, index=[8, 10], name="c", dtype="int64")
         result = df.loc[4, "c"]
         tm.assert_series_equal(result, exp)
-        if not using_array_manager:
-            # FIXME(ArrayManager): this correctly preserves dtype,
-            #  but incorrectly is not inplace.
-            # extra check for inplace-ness
+
+        # extra check for inplace-ness
+        if not using_copy_on_write:
             tm.assert_numpy_array_equal(view, exp.values)
 
         # arr + 0.5 cannot be cast losslessly to int, so we upcast
@@ -407,16 +406,23 @@ class TestMultiIndexSetItem:
         reindexed = dft.reindex(columns=[("foo", "two")])
         tm.assert_series_equal(reindexed["foo", "two"], s > s.median())
 
-    def test_set_column_scalar_with_loc(self, multiindex_dataframe_random_data):
+    def test_set_column_scalar_with_loc(
+        self, multiindex_dataframe_random_data, using_copy_on_write
+    ):
         frame = multiindex_dataframe_random_data
         subset = frame.index[[1, 4, 5]]
 
         frame.loc[subset] = 99
         assert (frame.loc[subset].values == 99).all()
 
+        frame_original = frame.copy()
         col = frame["B"]
         col[subset] = 97
-        assert (frame.loc[subset, "B"] == 97).all()
+        if using_copy_on_write:
+            # chained setitem doesn't work with CoW
+            tm.assert_frame_equal(frame, frame_original)
+        else:
+            assert (frame.loc[subset, "B"] == 97).all()
 
     def test_nonunique_assignment_1750(self):
         df = DataFrame(
@@ -489,21 +495,32 @@ def test_frame_setitem_view_direct(multiindex_dataframe_random_data):
     assert (df["foo"].values == 0).all()
 
 
-def test_frame_setitem_copy_raises(multiindex_dataframe_random_data):
+def test_frame_setitem_copy_raises(
+    multiindex_dataframe_random_data, using_copy_on_write
+):
     # will raise/warn as its chained assignment
     df = multiindex_dataframe_random_data.T
-    msg = "A value is trying to be set on a copy of a slice from a DataFrame"
-    with pytest.raises(com.SettingWithCopyError, match=msg):
+    if using_copy_on_write:
+        # TODO(CoW) it would be nice if this could still warn/raise
         df["foo"]["one"] = 2
+    else:
+        msg = "A value is trying to be set on a copy of a slice from a DataFrame"
+        with pytest.raises(SettingWithCopyError, match=msg):
+            df["foo"]["one"] = 2
 
 
-def test_frame_setitem_copy_no_write(multiindex_dataframe_random_data):
+def test_frame_setitem_copy_no_write(
+    multiindex_dataframe_random_data, using_copy_on_write
+):
     frame = multiindex_dataframe_random_data.T
     expected = frame
     df = frame.copy()
-    msg = "A value is trying to be set on a copy of a slice from a DataFrame"
-    with pytest.raises(com.SettingWithCopyError, match=msg):
+    if using_copy_on_write:
         df["foo"]["one"] = 2
+    else:
+        msg = "A value is trying to be set on a copy of a slice from a DataFrame"
+        with pytest.raises(SettingWithCopyError, match=msg):
+            df["foo"]["one"] = 2
 
     result = df
     tm.assert_frame_equal(result, expected)
