@@ -5,10 +5,12 @@ from textwrap import dedent
 import numpy as np
 import pytest
 
-import pandas as pd
 from pandas import (
     DataFrame,
+    IndexSlice,
     MultiIndex,
+    Series,
+    option_context,
 )
 import pandas._testing as tm
 
@@ -53,6 +55,9 @@ def mi_styler_comp(mi_styler):
     mi_styler.hide(axis="index")
     mi_styler.hide([("i0", "i1_a")], axis="index", names=True)
     mi_styler.set_table_attributes('class="box"')
+    other = mi_styler.data.agg(["mean"])
+    other.index = MultiIndex.from_product([[""], other.index])
+    mi_styler.concat(other.style)
     mi_styler.format(na_rep="MISSING", precision=3)
     mi_styler.format_index(precision=2, axis=0)
     mi_styler.format_index(precision=4, axis=1)
@@ -74,6 +79,25 @@ def mi_styler_comp(mi_styler):
     return mi_styler
 
 
+@pytest.fixture
+def blank_value():
+    return "&nbsp;"
+
+
+@pytest.fixture
+def df():
+    np.random.seed(24)
+    df = DataFrame({"A": [0, 1], "B": np.random.randn(2)})
+    return df
+
+
+@pytest.fixture
+def styler(df):
+    np.random.seed(24)
+    df = DataFrame({"A": [0, 1], "B": np.random.randn(2)})
+    return Styler(df)
+
+
 @pytest.mark.parametrize(
     "sparse_columns, exp_cols",
     [
@@ -137,16 +161,16 @@ def test_mi_styler_sparsify_index(mi_styler, sparse_index, exp_rows):
 
 
 def test_mi_styler_sparsify_options(mi_styler):
-    with pd.option_context("styler.sparse.index", False):
+    with option_context("styler.sparse.index", False):
         html1 = mi_styler.to_html()
-    with pd.option_context("styler.sparse.index", True):
+    with option_context("styler.sparse.index", True):
         html2 = mi_styler.to_html()
 
     assert html1 != html2
 
-    with pd.option_context("styler.sparse.columns", False):
+    with option_context("styler.sparse.columns", False):
         html1 = mi_styler.to_html()
-    with pd.option_context("styler.sparse.columns", True):
+    with option_context("styler.sparse.columns", True):
         html2 = mi_styler.to_html()
 
     assert html1 != html2
@@ -180,7 +204,7 @@ def test_trimming_maximum(rn, cn, max_els, max_rows, max_cols, exp_rn, exp_cn):
 def test_render_trimming_rows(option, val):
     # test auto and specific trimming of rows
     df = DataFrame(np.arange(120).reshape(60, 2))
-    with pd.option_context(option, val):
+    with option_context(option, val):
         ctx = df.style._translate(True, True)
     assert len(ctx["head"][0]) == 3  # index + 2 data cols
     assert len(ctx["body"]) == 4  # 3 data rows + trimming row
@@ -197,7 +221,7 @@ def test_render_trimming_rows(option, val):
 def test_render_trimming_cols(option, val):
     # test auto and specific trimming of cols
     df = DataFrame(np.arange(30).reshape(3, 10))
-    with pd.option_context(option, val):
+    with option_context(option, val):
         ctx = df.style._translate(True, True)
     assert len(ctx["head"][0]) == 4  # index + 2 data cols + trimming col
     assert len(ctx["body"]) == 3  # 3 data rows
@@ -207,7 +231,7 @@ def test_render_trimming_cols(option, val):
 def test_render_trimming_mi():
     midx = MultiIndex.from_product([[1, 2], [1, 2, 3]])
     df = DataFrame(np.arange(36).reshape(6, 6), columns=midx, index=midx)
-    with pd.option_context("styler.render.max_elements", 4):
+    with option_context("styler.render.max_elements", 4):
         ctx = df.style._translate(True, True)
 
     assert len(ctx["body"][0]) == 5  # 2 indexes + 2 data cols + trimming row
@@ -438,70 +462,49 @@ def test_apply_map_header_raises(mi_styler):
 
 
 class TestStyler:
-    def setup_method(self, method):
-        np.random.seed(24)
-        self.s = DataFrame({"A": np.random.permutation(range(6))})
-        self.df = DataFrame({"A": [0, 1], "B": np.random.randn(2)})
-        self.f = lambda x: x
-        self.g = lambda x: x
-
-        def h(x, foo="bar"):
-            return pd.Series(f"color: {foo}", index=x.index, name=x.name)
-
-        self.h = h
-        self.styler = Styler(self.df)
-        self.attrs = DataFrame({"A": ["color: red", "color: blue"]})
-        self.dataframes = [
-            self.df,
-            DataFrame(
-                {"f": [1.0, 2.0], "o": ["a", "b"], "c": pd.Categorical(["a", "b"])}
-            ),
-        ]
-        self.blank_value = "&nbsp;"
-
     def test_init_non_pandas(self):
         msg = "``data`` must be a Series or DataFrame"
         with pytest.raises(TypeError, match=msg):
             Styler([1, 2, 3])
 
     def test_init_series(self):
-        result = Styler(pd.Series([1, 2]))
+        result = Styler(Series([1, 2]))
         assert result.data.ndim == 2
 
-    def test_repr_html_ok(self):
-        self.styler._repr_html_()
+    def test_repr_html_ok(self, styler):
+        styler._repr_html_()
 
-    def test_repr_html_mathjax(self):
+    def test_repr_html_mathjax(self, styler):
         # gh-19824 / 41395
-        assert "tex2jax_ignore" not in self.styler._repr_html_()
+        assert "tex2jax_ignore" not in styler._repr_html_()
 
-        with pd.option_context("styler.html.mathjax", False):
-            assert "tex2jax_ignore" in self.styler._repr_html_()
+        with option_context("styler.html.mathjax", False):
+            assert "tex2jax_ignore" in styler._repr_html_()
 
-    def test_update_ctx(self):
-        self.styler._update_ctx(self.attrs)
+    def test_update_ctx(self, styler):
+        styler._update_ctx(DataFrame({"A": ["color: red", "color: blue"]}))
         expected = {(0, 0): [("color", "red")], (1, 0): [("color", "blue")]}
-        assert self.styler.ctx == expected
+        assert styler.ctx == expected
 
-    def test_update_ctx_flatten_multi_and_trailing_semi(self):
+    def test_update_ctx_flatten_multi_and_trailing_semi(self, styler):
         attrs = DataFrame({"A": ["color: red; foo: bar", "color:blue ; foo: baz;"]})
-        self.styler._update_ctx(attrs)
+        styler._update_ctx(attrs)
         expected = {
             (0, 0): [("color", "red"), ("foo", "bar")],
             (1, 0): [("color", "blue"), ("foo", "baz")],
         }
-        assert self.styler.ctx == expected
+        assert styler.ctx == expected
 
     def test_render(self):
         df = DataFrame({"A": [0, 1]})
-        style = lambda x: pd.Series(["color: red", "color: blue"], name=x.name)
+        style = lambda x: Series(["color: red", "color: blue"], name=x.name)
         s = Styler(df, uuid="AB").apply(style)
         s.to_html()
         # it worked?
 
-    def test_multiple_render(self):
+    def test_multiple_render(self, df):
         # GH 39396
-        s = Styler(self.df, uuid_len=0).applymap(lambda x: "color: red;", subset=["A"])
+        s = Styler(df, uuid_len=0).applymap(lambda x: "color: red;", subset=["A"])
         s.to_html()  # do 2 renders to ensure css styles not duplicated
         assert (
             '<style type="text/css">\n#T__row0_col0, #T__row1_col0 {\n'
@@ -520,7 +523,7 @@ class TestStyler:
 
     def test_render_double(self):
         df = DataFrame({"A": [0, 1]})
-        style = lambda x: pd.Series(
+        style = lambda x: Series(
             ["color: red; border: 1px", "color: blue; border: 2px"], name=x.name
         )
         s = Styler(df, uuid="AB").apply(style)
@@ -540,14 +543,14 @@ class TestStyler:
     def test_set_properties_subset(self):
         df = DataFrame({"A": [0, 1]})
         result = (
-            df.style.set_properties(subset=pd.IndexSlice[0, "A"], color="white")
+            df.style.set_properties(subset=IndexSlice[0, "A"], color="white")
             ._compute()
             .ctx
         )
         expected = {(0, 0): [("color", "white")]}
         assert result == expected
 
-    def test_empty_index_name_doesnt_display(self):
+    def test_empty_index_name_doesnt_display(self, blank_value):
         # https://github.com/pandas-dev/pandas/pull/12090#issuecomment-180695902
         df = DataFrame({"A": [1, 2], "B": [3, 4], "C": [5, 6]})
         result = df.style._translate(True, True)
@@ -555,9 +558,9 @@ class TestStyler:
         expected = {
             "class": "blank level0",
             "type": "th",
-            "value": self.blank_value,
+            "value": blank_value,
             "is_visible": True,
-            "display_value": self.blank_value,
+            "display_value": blank_value,
         }
         assert expected.items() <= result["head"][0][0].items()
 
@@ -614,13 +617,13 @@ class TestStyler:
         df = DataFrame([[1, 2], [3, 4]], index=["X", "Y"], columns=["X", "Y"])
 
         # test Series return where len(Series) < df.index or df.columns but labels OK
-        func = lambda s: pd.Series(["color: red;"], index=["Y"])
+        func = lambda s: Series(["color: red;"], index=["Y"])
         result = df.style.apply(func, axis=axis)._compute().ctx
         assert result[(1, 1)] == [("color", "red")]
         assert result[(1 - axis, axis)] == [("color", "red")]
 
         # test Series return where labels align but different order
-        func = lambda s: pd.Series(["color: red;", "color: blue;"], index=["Y", "X"])
+        func = lambda s: Series(["color: red;", "color: blue;"], index=["Y", "X"])
         result = df.style.apply(func, axis=axis)._compute().ctx
         assert result[(0, 0)] == [("color", "blue")]
         assert result[(1, 1)] == [("color", "red")]
@@ -645,61 +648,58 @@ class TestStyler:
     @pytest.mark.parametrize(
         "slice_",
         [
-            pd.IndexSlice[:],
-            pd.IndexSlice[:, ["A"]],
-            pd.IndexSlice[[1], :],
-            pd.IndexSlice[[1], ["A"]],
-            pd.IndexSlice[:2, ["A", "B"]],
+            IndexSlice[:],
+            IndexSlice[:, ["A"]],
+            IndexSlice[[1], :],
+            IndexSlice[[1], ["A"]],
+            IndexSlice[:2, ["A", "B"]],
         ],
     )
     @pytest.mark.parametrize("axis", [0, 1])
-    def test_apply_subset(self, slice_, axis):
-        result = (
-            self.df.style.apply(self.h, axis=axis, subset=slice_, foo="baz")
-            ._compute()
-            .ctx
-        )
+    def test_apply_subset(self, slice_, axis, df):
+        def h(x, foo="bar"):
+            return Series(f"color: {foo}", index=x.index, name=x.name)
+
+        result = df.style.apply(h, axis=axis, subset=slice_, foo="baz")._compute().ctx
         expected = {
             (r, c): [("color", "baz")]
-            for r, row in enumerate(self.df.index)
-            for c, col in enumerate(self.df.columns)
-            if row in self.df.loc[slice_].index and col in self.df.loc[slice_].columns
+            for r, row in enumerate(df.index)
+            for c, col in enumerate(df.columns)
+            if row in df.loc[slice_].index and col in df.loc[slice_].columns
         }
         assert result == expected
 
     @pytest.mark.parametrize(
         "slice_",
         [
-            pd.IndexSlice[:],
-            pd.IndexSlice[:, ["A"]],
-            pd.IndexSlice[[1], :],
-            pd.IndexSlice[[1], ["A"]],
-            pd.IndexSlice[:2, ["A", "B"]],
+            IndexSlice[:],
+            IndexSlice[:, ["A"]],
+            IndexSlice[[1], :],
+            IndexSlice[[1], ["A"]],
+            IndexSlice[:2, ["A", "B"]],
         ],
     )
-    def test_applymap_subset(self, slice_):
-        result = (
-            self.df.style.applymap(lambda x: "color:baz;", subset=slice_)._compute().ctx
-        )
+    def test_applymap_subset(self, slice_, df):
+        result = df.style.applymap(lambda x: "color:baz;", subset=slice_)._compute().ctx
         expected = {
             (r, c): [("color", "baz")]
-            for r, row in enumerate(self.df.index)
-            for c, col in enumerate(self.df.columns)
-            if row in self.df.loc[slice_].index and col in self.df.loc[slice_].columns
+            for r, row in enumerate(df.index)
+            for c, col in enumerate(df.columns)
+            if row in df.loc[slice_].index and col in df.loc[slice_].columns
         }
         assert result == expected
 
     @pytest.mark.parametrize(
         "slice_",
         [
-            pd.IndexSlice[:, pd.IndexSlice["x", "A"]],
-            pd.IndexSlice[:, pd.IndexSlice[:, "A"]],
-            pd.IndexSlice[:, pd.IndexSlice[:, ["A", "C"]]],  # missing col element
-            pd.IndexSlice[pd.IndexSlice["a", 1], :],
-            pd.IndexSlice[pd.IndexSlice[:, 1], :],
-            pd.IndexSlice[pd.IndexSlice[:, [1, 3]], :],  # missing row element
-            pd.IndexSlice[:, ("x", "A")],
-            pd.IndexSlice[("a", 1), :],
+            IndexSlice[:, IndexSlice["x", "A"]],
+            IndexSlice[:, IndexSlice[:, "A"]],
+            IndexSlice[:, IndexSlice[:, ["A", "C"]]],  # missing col element
+            IndexSlice[IndexSlice["a", 1], :],
+            IndexSlice[IndexSlice[:, 1], :],
+            IndexSlice[IndexSlice[:, [1, 3]], :],  # missing row element
+            IndexSlice[:, ("x", "A")],
+            IndexSlice[("a", 1), :],
         ],
     )
     def test_applymap_subset_multiindex(self, slice_):
@@ -737,7 +737,7 @@ class TestStyler:
         df = DataFrame(
             [[1, -1, 1, 1], [-1, 1, 1, 1]], index=["hello", "world"], columns=columns
         )
-        pct_subset = pd.IndexSlice[:, pd.IndexSlice[:, "%":"%"]]
+        pct_subset = IndexSlice[:, IndexSlice[:, "%":"%"]]
 
         def color_negative_red(val):
             color = "red" if val < 0 else "black"
@@ -746,6 +746,22 @@ class TestStyler:
         df.loc[pct_subset]
         df.style.applymap(color_negative_red, subset=pct_subset)
 
+    @pytest.mark.parametrize(
+        "stylefunc", ["background_gradient", "bar", "text_gradient"]
+    )
+    def test_subset_for_boolean_cols(self, stylefunc):
+        # GH47838
+        df = DataFrame(
+            [
+                [1, 2],
+                [3, 4],
+            ],
+            columns=[False, True],
+        )
+        styled = getattr(df.style, stylefunc)()
+        styled._compute()
+        assert set(styled.ctx) == {(0, 0), (0, 1), (1, 0), (1, 1)}
+
     def test_empty(self):
         df = DataFrame({"A": [1, 0]})
         s = df.style
@@ -777,22 +793,22 @@ class TestStyler:
         assert ctx["body"][0][1]["display_value"] == "NA"
         assert ctx["body"][0][2]["display_value"] == "NA"
 
-    def test_caption(self):
-        styler = Styler(self.df, caption="foo")
+    def test_caption(self, df):
+        styler = Styler(df, caption="foo")
         result = styler.to_html()
         assert all(["caption" in result, "foo" in result])
 
-        styler = self.df.style
+        styler = df.style
         result = styler.set_caption("baz")
         assert styler is result
         assert styler.caption == "baz"
 
-    def test_uuid(self):
-        styler = Styler(self.df, uuid="abc123")
+    def test_uuid(self, df):
+        styler = Styler(df, uuid="abc123")
         result = styler.to_html()
         assert "abc123" in result
 
-        styler = self.df.style
+        styler = df.style
         result = styler.set_uuid("aaa")
         assert result is styler
         assert result.uuid == "aaa"
@@ -805,25 +821,25 @@ class TestStyler:
         ids = re.findall('id="(.*?)"', result)
         assert np.unique(ids).size == len(ids)
 
-    def test_table_styles(self):
+    def test_table_styles(self, df):
         style = [{"selector": "th", "props": [("foo", "bar")]}]  # default format
-        styler = Styler(self.df, table_styles=style)
+        styler = Styler(df, table_styles=style)
         result = " ".join(styler.to_html().split())
         assert "th { foo: bar; }" in result
 
-        styler = self.df.style
+        styler = df.style
         result = styler.set_table_styles(style)
         assert styler is result
         assert styler.table_styles == style
 
         # GH 39563
         style = [{"selector": "th", "props": "foo:bar;"}]  # css string format
-        styler = self.df.style.set_table_styles(style)
+        styler = df.style.set_table_styles(style)
         result = " ".join(styler.to_html().split())
         assert "th { foo: bar; }" in result
 
-    def test_table_styles_multiple(self):
-        ctx = self.df.style.set_table_styles(
+    def test_table_styles_multiple(self, df):
+        ctx = df.style.set_table_styles(
             [
                 {"selector": "th,td", "props": "color:red;"},
                 {"selector": "tr", "props": "color:green;"},
@@ -835,15 +851,19 @@ class TestStyler:
             {"selector": "tr", "props": [("color", "green")]},
         ]
 
-    def test_table_styles_dict_multiple_selectors(self):
+    def test_table_styles_dict_multiple_selectors(self, df):
         # GH 44011
-        result = self.df.style.set_table_styles(
-            [{"selector": "th,td", "props": [("border-left", "2px solid black")]}]
+        result = df.style.set_table_styles(
+            {
+                "B": [
+                    {"selector": "th,td", "props": [("border-left", "2px solid black")]}
+                ]
+            }
         )._translate(True, True)["table_styles"]
 
         expected = [
-            {"selector": "th", "props": [("border-left", "2px solid black")]},
-            {"selector": "td", "props": [("border-left", "2px solid black")]},
+            {"selector": "th.col1", "props": [("border-left", "2px solid black")]},
+            {"selector": "td.col1", "props": [("border-left", "2px solid black")]},
         ]
 
         assert result == expected
@@ -860,13 +880,13 @@ class TestStyler:
         with pytest.raises(ValueError, match=msg):
             maybe_convert_css_to_tuples("err")
 
-    def test_table_attributes(self):
+    def test_table_attributes(self, df):
         attributes = 'class="foo" data-bar'
-        styler = Styler(self.df, table_attributes=attributes)
+        styler = Styler(df, table_attributes=attributes)
         result = styler.to_html()
         assert 'class="foo" data-bar' in result
 
-        result = self.df.style.set_table_attributes(attributes).to_html()
+        result = df.style.set_table_attributes(attributes).to_html()
         assert 'class="foo" data-bar' in result
 
     def test_apply_none(self):
@@ -880,20 +900,20 @@ class TestStyler:
         result = DataFrame([[1, 2], [3, 4]]).style.apply(f, axis=None)._compute().ctx
         assert result[(1, 1)] == [("color", "red")]
 
-    def test_trim(self):
-        result = self.df.style.to_html()  # trim=True
+    def test_trim(self, df):
+        result = df.style.to_html()  # trim=True
         assert result.count("#") == 0
 
-        result = self.df.style.highlight_max().to_html()
-        assert result.count("#") == len(self.df.columns)
+        result = df.style.highlight_max().to_html()
+        assert result.count("#") == len(df.columns)
 
-    def test_export(self):
+    def test_export(self, df, styler):
         f = lambda x: "color: red" if x > 0 else "color: blue"
         g = lambda x, z: f"color: {z}" if x > 0 else f"color: {z}"
-        style1 = self.styler
+        style1 = styler
         style1.applymap(f).applymap(g, z="b").highlight_max()._compute()  # = render
         result = style1.export()
-        style2 = self.df.style
+        style2 = df.style
         style2.use(result)
         assert style1._todo == style2._todo
         style2.to_html()
@@ -913,13 +933,13 @@ class TestStyler:
             df.style._apply(lambda x: ["", "", "", ""])
 
         with pytest.raises(ValueError, match=msg.format("index")):
-            df.style._apply(lambda x: pd.Series(["a:v;", ""], index=["A", "C"]), axis=0)
+            df.style._apply(lambda x: Series(["a:v;", ""], index=["A", "C"]), axis=0)
 
         with pytest.raises(ValueError, match=msg.format("columns")):
             df.style._apply(lambda x: ["", "", ""], axis=1)
 
         with pytest.raises(ValueError, match=msg.format("columns")):
-            df.style._apply(lambda x: pd.Series(["a:v;", ""], index=["X", "Z"]), axis=1)
+            df.style._apply(lambda x: Series(["a:v;", ""], index=["X", "Z"]), axis=1)
 
         msg = "returned ndarray with wrong shape"
         with pytest.raises(ValueError, match=msg):
@@ -1006,7 +1026,7 @@ class TestStyler:
         result = _get_level_lengths(index, sparsify=False, max_index=100)
         tm.assert_dict_equal(result, expected)
 
-    def test_mi_sparse_index_names(self):
+    def test_mi_sparse_index_names(self, blank_value):
         # Test the class names and displayed value are correct on rendering MI names
         df = DataFrame(
             {"A": [1, 2]},
@@ -1029,14 +1049,14 @@ class TestStyler:
             },
             {
                 "class": "blank col0",
-                "display_value": self.blank_value,
+                "display_value": blank_value,
                 "is_visible": True,
             },
         ]
         for i, expected_dict in enumerate(expected):
             assert expected_dict.items() <= head[i].items()
 
-    def test_mi_sparse_column_names(self):
+    def test_mi_sparse_column_names(self, blank_value):
         df = DataFrame(
             np.arange(16).reshape(4, 4),
             index=MultiIndex.from_arrays(
@@ -1054,7 +1074,7 @@ class TestStyler:
             expected = [
                 {
                     "class": "blank",
-                    "display_value": self.blank_value,
+                    "display_value": blank_value,
                     "is_visible": True,
                 },
                 {
@@ -1066,32 +1086,32 @@ class TestStyler:
             for i, expected_dict in enumerate(expected):
                 assert expected_dict.items() <= head[i].items()
 
-    def test_hide_column_headers(self):
-        ctx = self.styler.hide(axis="columns")._translate(True, True)
+    def test_hide_column_headers(self, df, styler):
+        ctx = styler.hide(axis="columns")._translate(True, True)
         assert len(ctx["head"]) == 0  # no header entries with an unnamed index
 
-        self.df.index.name = "some_name"
-        ctx = self.df.style.hide(axis="columns")._translate(True, True)
+        df.index.name = "some_name"
+        ctx = df.style.hide(axis="columns")._translate(True, True)
         assert len(ctx["head"]) == 1
         # index names still visible, changed in #42101, reverted in 43404
 
-    def test_hide_single_index(self):
+    def test_hide_single_index(self, df):
         # GH 14194
         # single unnamed index
-        ctx = self.df.style._translate(True, True)
+        ctx = df.style._translate(True, True)
         assert ctx["body"][0][0]["is_visible"]
         assert ctx["head"][0][0]["is_visible"]
-        ctx2 = self.df.style.hide(axis="index")._translate(True, True)
+        ctx2 = df.style.hide(axis="index")._translate(True, True)
         assert not ctx2["body"][0][0]["is_visible"]
         assert not ctx2["head"][0][0]["is_visible"]
 
         # single named index
-        ctx3 = self.df.set_index("A").style._translate(True, True)
+        ctx3 = df.set_index("A").style._translate(True, True)
         assert ctx3["body"][0][0]["is_visible"]
         assert len(ctx3["head"]) == 2  # 2 header levels
         assert ctx3["head"][0][0]["is_visible"]
 
-        ctx4 = self.df.set_index("A").style.hide(axis="index")._translate(True, True)
+        ctx4 = df.set_index("A").style.hide(axis="index")._translate(True, True)
         assert not ctx4["body"][0][0]["is_visible"]
         assert len(ctx4["head"]) == 1  # only 1 header levels
         assert not ctx4["head"][0][0]["is_visible"]
@@ -1119,10 +1139,10 @@ class TestStyler:
         assert len(ctx2["head"][0]) == 3  # one hidden (col name) and two data columns
         assert not ctx2["head"][0][0]["is_visible"]
 
-    def test_hide_columns_single_level(self):
+    def test_hide_columns_single_level(self, df):
         # GH 14194
         # test hiding single column
-        ctx = self.df.style._translate(True, True)
+        ctx = df.style._translate(True, True)
         assert ctx["head"][0][1]["is_visible"]
         assert ctx["head"][0][1]["display_value"] == "A"
         assert ctx["head"][0][2]["is_visible"]
@@ -1130,13 +1150,13 @@ class TestStyler:
         assert ctx["body"][0][1]["is_visible"]  # col A, row 1
         assert ctx["body"][1][2]["is_visible"]  # col B, row 1
 
-        ctx = self.df.style.hide("A", axis="columns")._translate(True, True)
+        ctx = df.style.hide("A", axis="columns")._translate(True, True)
         assert not ctx["head"][0][1]["is_visible"]
         assert not ctx["body"][0][1]["is_visible"]  # col A, row 1
         assert ctx["body"][1][2]["is_visible"]  # col B, row 1
 
         # test hiding mulitiple columns
-        ctx = self.df.style.hide(["A", "B"], axis="columns")._translate(True, True)
+        ctx = df.style.hide(["A", "B"], axis="columns")._translate(True, True)
         assert not ctx["head"][0][1]["is_visible"]
         assert not ctx["head"][0][2]["is_visible"]
         assert not ctx["body"][0][1]["is_visible"]  # col A, row 1
@@ -1202,18 +1222,18 @@ class TestStyler:
             assert "row1" in ctx["body"][0][i]["class"]  # row0 not included in body
             assert ctx["body"][0][i]["is_visible"]
 
-    def test_pipe(self):
+    def test_pipe(self, df):
         def set_caption_from_template(styler, a, b):
             return styler.set_caption(f"Dataframe with a = {a} and b = {b}")
 
-        styler = self.df.style.pipe(set_caption_from_template, "A", b="B")
+        styler = df.style.pipe(set_caption_from_template, "A", b="B")
         assert "Dataframe with a = A and b = B" in styler.to_html()
 
         # Test with an argument that is a (callable, keyword_name) pair.
         def f(a, b, styler):
             return (a, b, styler)
 
-        styler = self.df.style
+        styler = df.style
         result = styler.pipe((f, "styler"), a=1, b=2)
         assert result == (1, 2, styler)
 
@@ -1315,18 +1335,18 @@ class TestStyler:
     @pytest.mark.parametrize(
         "slc",
         [
-            pd.IndexSlice[:, :],
-            pd.IndexSlice[:, 1],
-            pd.IndexSlice[1, :],
-            pd.IndexSlice[[1], [1]],
-            pd.IndexSlice[1, [1]],
-            pd.IndexSlice[[1], 1],
-            pd.IndexSlice[1],
-            pd.IndexSlice[1, 1],
+            IndexSlice[:, :],
+            IndexSlice[:, 1],
+            IndexSlice[1, :],
+            IndexSlice[[1], [1]],
+            IndexSlice[1, [1]],
+            IndexSlice[[1], 1],
+            IndexSlice[1],
+            IndexSlice[1, 1],
             slice(None, None, None),
             [0, 1],
             np.array([0, 1]),
-            pd.Series([0, 1]),
+            Series([0, 1]),
         ],
     )
     def test_non_reducing_slice(self, slc):
@@ -1335,13 +1355,13 @@ class TestStyler:
         tslice_ = non_reducing_slice(slc)
         assert isinstance(df.loc[tslice_], DataFrame)
 
-    @pytest.mark.parametrize("box", [list, pd.Series, np.array])
+    @pytest.mark.parametrize("box", [list, Series, np.array])
     def test_list_slice(self, box):
         # like dataframe getitem
         subset = box(["A"])
 
         df = DataFrame({"A": [1, 2], "B": [3, 4]}, index=["A", "B"])
-        expected = pd.IndexSlice[:, ["A"]]
+        expected = IndexSlice[:, ["A"]]
 
         result = non_reducing_slice(subset)
         tm.assert_frame_equal(df.loc[result], df.loc[expected])
@@ -1355,7 +1375,7 @@ class TestStyler:
             ("b", "d"): [4, 1],
         }
         df = DataFrame(dic, index=[0, 1])
-        idx = pd.IndexSlice
+        idx = IndexSlice
         slice_ = idx[:, idx["b", "d"]]
         tslice_ = non_reducing_slice(slice_)
 
@@ -1366,27 +1386,27 @@ class TestStyler:
     @pytest.mark.parametrize(
         "slice_",
         [
-            pd.IndexSlice[:, :],
+            IndexSlice[:, :],
             # check cols
-            pd.IndexSlice[:, pd.IndexSlice[["a"]]],  # inferred deeper need list
-            pd.IndexSlice[:, pd.IndexSlice[["a"], ["c"]]],  # inferred deeper need list
-            pd.IndexSlice[:, pd.IndexSlice["a", "c", :]],
-            pd.IndexSlice[:, pd.IndexSlice["a", :, "e"]],
-            pd.IndexSlice[:, pd.IndexSlice[:, "c", "e"]],
-            pd.IndexSlice[:, pd.IndexSlice["a", ["c", "d"], :]],  # check list
-            pd.IndexSlice[:, pd.IndexSlice["a", ["c", "d", "-"], :]],  # allow missing
-            pd.IndexSlice[:, pd.IndexSlice["a", ["c", "d", "-"], "e"]],  # no slice
+            IndexSlice[:, IndexSlice[["a"]]],  # inferred deeper need list
+            IndexSlice[:, IndexSlice[["a"], ["c"]]],  # inferred deeper need list
+            IndexSlice[:, IndexSlice["a", "c", :]],
+            IndexSlice[:, IndexSlice["a", :, "e"]],
+            IndexSlice[:, IndexSlice[:, "c", "e"]],
+            IndexSlice[:, IndexSlice["a", ["c", "d"], :]],  # check list
+            IndexSlice[:, IndexSlice["a", ["c", "d", "-"], :]],  # allow missing
+            IndexSlice[:, IndexSlice["a", ["c", "d", "-"], "e"]],  # no slice
             # check rows
-            pd.IndexSlice[pd.IndexSlice[["U"]], :],  # inferred deeper need list
-            pd.IndexSlice[pd.IndexSlice[["U"], ["W"]], :],  # inferred deeper need list
-            pd.IndexSlice[pd.IndexSlice["U", "W", :], :],
-            pd.IndexSlice[pd.IndexSlice["U", :, "Y"], :],
-            pd.IndexSlice[pd.IndexSlice[:, "W", "Y"], :],
-            pd.IndexSlice[pd.IndexSlice[:, "W", ["Y", "Z"]], :],  # check list
-            pd.IndexSlice[pd.IndexSlice[:, "W", ["Y", "Z", "-"]], :],  # allow missing
-            pd.IndexSlice[pd.IndexSlice["U", "W", ["Y", "Z", "-"]], :],  # no slice
+            IndexSlice[IndexSlice[["U"]], :],  # inferred deeper need list
+            IndexSlice[IndexSlice[["U"], ["W"]], :],  # inferred deeper need list
+            IndexSlice[IndexSlice["U", "W", :], :],
+            IndexSlice[IndexSlice["U", :, "Y"], :],
+            IndexSlice[IndexSlice[:, "W", "Y"], :],
+            IndexSlice[IndexSlice[:, "W", ["Y", "Z"]], :],  # check list
+            IndexSlice[IndexSlice[:, "W", ["Y", "Z", "-"]], :],  # allow missing
+            IndexSlice[IndexSlice["U", "W", ["Y", "Z", "-"]], :],  # no slice
             # check simultaneous
-            pd.IndexSlice[pd.IndexSlice[:, "W", "Y"], pd.IndexSlice["a", "c", :]],
+            IndexSlice[IndexSlice[:, "W", "Y"], IndexSlice["a", "c", :]],
         ],
     )
     def test_non_reducing_multi_slice_on_multiindex(self, slice_):
@@ -1505,7 +1525,7 @@ def test_get_level_lengths_mi_hidden():
 def test_row_trimming_hide_index():
     # gh 43703
     df = DataFrame([[1], [2], [3], [4], [5]])
-    with pd.option_context("styler.render.max_rows", 2):
+    with option_context("styler.render.max_rows", 2):
         ctx = df.style.hide([0, 1], axis="index")._translate(True, True)
     assert len(ctx["body"]) == 3
     for r, val in enumerate(["3", "4", "..."]):
@@ -1516,7 +1536,7 @@ def test_row_trimming_hide_index_mi():
     # gh 44247
     df = DataFrame([[1], [2], [3], [4], [5]])
     df.index = MultiIndex.from_product([[0], [0, 1, 2, 3, 4]])
-    with pd.option_context("styler.render.max_rows", 2):
+    with option_context("styler.render.max_rows", 2):
         ctx = df.style.hide([(0, 0), (0, 1)], axis="index")._translate(True, True)
     assert len(ctx["body"]) == 3
 
@@ -1538,7 +1558,7 @@ def test_row_trimming_hide_index_mi():
 def test_col_trimming_hide_columns():
     # gh 44272
     df = DataFrame([[1, 2, 3, 4, 5]])
-    with pd.option_context("styler.render.max_columns", 2):
+    with option_context("styler.render.max_columns", 2):
         ctx = df.style.hide([0, 1], axis="columns")._translate(True, True)
 
     assert len(ctx["head"][0]) == 6  # blank, [0, 1 (hidden)], [2 ,3 (visible)], + trim
@@ -1549,7 +1569,13 @@ def test_col_trimming_hide_columns():
     assert len(ctx["body"][0]) == 6  # index + 2 hidden + 2 visible + trimming col
 
 
-@pytest.mark.parametrize("format", ["html", "latex"])
+def test_no_empty_apply(mi_styler):
+    # 45313
+    mi_styler.apply(lambda s: ["a:v;"] * 2, subset=[False, False])
+    mi_styler._compute()
+
+
+@pytest.mark.parametrize("format", ["html", "latex", "string"])
 def test_output_buffer(mi_styler, format):
     # gh 47053
     with tm.ensure_clean(f"delete_me.{format}") as f:
