@@ -3,7 +3,10 @@ from datetime import datetime
 import numpy as np
 import pytest
 
-from pandas import DataFrame, Series
+from pandas import (
+    DataFrame,
+    Series,
+)
 from pandas.core.indexes.datetimes import date_range
 from pandas.core.indexes.period import period_range
 
@@ -34,12 +37,6 @@ def downsample_method(request):
     return request.param
 
 
-@pytest.fixture(params=upsample_methods)
-def upsample_method(request):
-    """Fixture for parametrization of Grouper upsample methods."""
-    return request.param
-
-
 @pytest.fixture(params=resample_methods)
 def resample_method(request):
     """Fixture for parametrization of Grouper resample methods."""
