@@ -61,10 +61,7 @@ class TestConcatAppendCommon:
         considering not-supported dtypes
         """
         if isinstance(obj, Index):
-            if label == "bool":
-                assert obj.dtype == "object"
-            else:
-                assert obj.dtype == label
+            assert obj.dtype == label
         elif isinstance(obj, Series):
             if label.startswith("period"):
                 assert obj.dtype == "Period[M]"
@@ -73,11 +70,18 @@ class TestConcatAppendCommon:
         else:
             raise ValueError
 
-    def test_dtypes(self, item):
+    @pytest.mark.parametrize("box", [Index, Series])
+    def test_dtypes(self, item, box):
         # to confirm test case covers intended dtypes
         typ, vals = item
-        self._check_expected_dtype(Index(vals), typ)
-        self._check_expected_dtype(Series(vals), typ)
+        obj = box(vals)
+        if isinstance(obj, Index):
+            assert obj.dtype == typ
+        elif isinstance(obj, Series):
+            if typ.startswith("period"):
+                assert obj.dtype == "Period[M]"
+            else:
+                assert obj.dtype == typ
 
     def test_concatlike_same_dtypes(self, item):
         # GH 13660
@@ -185,7 +189,7 @@ class TestConcatAppendCommon:
         with pytest.raises(TypeError, match=msg):
             pd.concat([Series(vals1), Series(vals2), vals3])
 
-    def test_concatlike_dtypes_coercion(self, item, item2):
+    def test_concatlike_dtypes_coercion(self, item, item2, request):
         # GH 13660
         typ1, vals1 = item
         typ2, vals2 = item2
@@ -210,9 +214,13 @@ class TestConcatAppendCommon:
             # series coerces to numeric based on numpy rule
             # index doesn't because bool is object dtype
             exp_series_dtype = typ2
+            mark = pytest.mark.xfail(reason="GH#39187 casting to object")
+            request.node.add_marker(mark)
             warn = FutureWarning
         elif typ2 == "bool" and typ1 in ("int64", "float64"):
             exp_series_dtype = typ1
+            mark = pytest.mark.xfail(reason="GH#39187 casting to object")
+            request.node.add_marker(mark)
             warn = FutureWarning
         elif (
             typ1 == "datetime64[ns, US/Eastern]"
@@ -229,7 +237,9 @@ class TestConcatAppendCommon:
         # ----- Index ----- #
 
         # index.append
-        res = Index(vals1).append(Index(vals2))
+        with tm.assert_produces_warning(warn, match="concatenating bool-dtype"):
+            # GH#39817
+            res = Index(vals1).append(Index(vals2))
         exp = Index(exp_data, dtype=exp_index_dtype)
         tm.assert_index_equal(res, exp)
 
@@ -508,7 +518,7 @@ class TestConcatAppendCommon:
         s1 = Series([10, 11, np.nan], dtype="category")
         s2 = Series([np.nan, 1, 3, 2], dtype="category")
 
-        exp = Series([10, 11, np.nan, np.nan, 1, 3, 2], dtype="object")
+        exp = Series([10, 11, np.nan, np.nan, 1, 3, 2], dtype=np.float64)
         tm.assert_series_equal(pd.concat([s1, s2], ignore_index=True), exp)
         tm.assert_series_equal(s1._append(s2, ignore_index=True), exp)
 
@@ -529,12 +539,12 @@ class TestConcatAppendCommon:
         s1 = Series([1, 2, np.nan], dtype="category")
         s2 = Series([2, 1, 2])
 
-        exp = Series([1, 2, np.nan, 2, 1, 2], dtype="object")
+        exp = Series([1, 2, np.nan, 2, 1, 2], dtype=np.float64)
         tm.assert_series_equal(pd.concat([s1, s2], ignore_index=True), exp)
         tm.assert_series_equal(s1._append(s2, ignore_index=True), exp)
 
         # result shouldn't be affected by 1st elem dtype
-        exp = Series([2, 1, 2, 1, 2, np.nan], dtype="object")
+        exp = Series([2, 1, 2, 1, 2, np.nan], dtype=np.float64)
         tm.assert_series_equal(pd.concat([s2, s1], ignore_index=True), exp)
         tm.assert_series_equal(s2._append(s1, ignore_index=True), exp)
 
@@ -554,11 +564,11 @@ class TestConcatAppendCommon:
         s1 = Series([10, 11, np.nan], dtype="category")
         s2 = Series([1, 3, 2])
 
-        exp = Series([10, 11, np.nan, 1, 3, 2], dtype="object")
+        exp = Series([10, 11, np.nan, 1, 3, 2], dtype=np.float64)
         tm.assert_series_equal(pd.concat([s1, s2], ignore_index=True), exp)
         tm.assert_series_equal(s1._append(s2, ignore_index=True), exp)
 
-        exp = Series([1, 3, 2, 10, 11, np.nan], dtype="object")
+        exp = Series([1, 3, 2, 10, 11, np.nan], dtype=np.float64)
         tm.assert_series_equal(pd.concat([s2, s1], ignore_index=True), exp)
         tm.assert_series_equal(s2._append(s1, ignore_index=True), exp)
 
