@@ -201,6 +201,12 @@ class TestSeriesGetitemScalars:
         with pytest.raises(KeyError, match=msg):
             ser["50 days"]
 
+    def test_getitem_bool_index_positional(self):
+        # GH#48653
+        ser = Series({True: 1, False: 0})
+        result = ser[0]
+        assert result == 1
+
 
 class TestSeriesGetitemSlices:
     def test_getitem_partial_str_slice_with_datetimeindex(self):
