@@ -15,11 +15,11 @@ import pandas._testing as tm
 
 class TestSeriesAppend:
     def test_append_preserve_name(self, datetime_series):
-        result = datetime_series[:5].append(datetime_series[5:])
+        result = datetime_series[:5]._append(datetime_series[5:])
         assert result.name == datetime_series.name
 
     def test_append(self, datetime_series, string_series, object_series):
-        appended_series = string_series.append(object_series)
+        appended_series = string_series._append(object_series)
         for idx, value in appended_series.items():
             if idx in string_series.index:
                 assert value == string_series[idx]
@@ -30,12 +30,12 @@ class TestSeriesAppend:
 
         msg = "Indexes have overlapping values:"
         with pytest.raises(ValueError, match=msg):
-            datetime_series.append(datetime_series, verify_integrity=True)
+            datetime_series._append(datetime_series, verify_integrity=True)
 
     def test_append_many(self, datetime_series):
         pieces = [datetime_series[:5], datetime_series[5:10], datetime_series[10:]]
 
-        result = pieces[0].append(pieces[1:])
+        result = pieces[0]._append(pieces[1:])
         tm.assert_series_equal(result, datetime_series)
 
     def test_append_duplicates(self):
@@ -43,13 +43,13 @@ class TestSeriesAppend:
         s1 = Series([1, 2, 3])
         s2 = Series([4, 5, 6])
         exp = Series([1, 2, 3, 4, 5, 6], index=[0, 1, 2, 0, 1, 2])
-        tm.assert_series_equal(s1.append(s2), exp)
+        tm.assert_series_equal(s1._append(s2), exp)
         tm.assert_series_equal(pd.concat([s1, s2]), exp)
 
         # the result must have RangeIndex
         exp = Series([1, 2, 3, 4, 5, 6])
         tm.assert_series_equal(
-            s1.append(s2, ignore_index=True), exp, check_index_type=True
+            s1._append(s2, ignore_index=True), exp, check_index_type=True
         )
         tm.assert_series_equal(
             pd.concat([s1, s2], ignore_index=True), exp, check_index_type=True
@@ -57,7 +57,7 @@ class TestSeriesAppend:
 
         msg = "Indexes have overlapping values:"
         with pytest.raises(ValueError, match=msg):
-            s1.append(s2, verify_integrity=True)
+            s1._append(s2, verify_integrity=True)
         with pytest.raises(ValueError, match=msg):
             pd.concat([s1, s2], verify_integrity=True)
 
@@ -67,8 +67,8 @@ class TestSeriesAppend:
         list_input = [s, s]
         tuple_input = (s, s)
 
-        expected = s.append(list_input)
-        result = s.append(tuple_input)
+        expected = s._append(list_input)
+        result = s._append(tuple_input)
 
         tm.assert_series_equal(expected, result)
 
@@ -78,9 +78,14 @@ class TestSeriesAppend:
 
         msg = "to_append should be a Series or list/tuple of Series, got DataFrame"
         with pytest.raises(TypeError, match=msg):
-            df.A.append(df)
+            df.A._append(df)
         with pytest.raises(TypeError, match=msg):
-            df.A.append([df])
+            df.A._append([df])
+
+    def test_append_raises_future_warning(self):
+        # GH#35407
+        with tm.assert_produces_warning(FutureWarning):
+            Series([1, 2]).append(Series([3, 4]))
 
 
 class TestSeriesAppendWithDatetimeIndex:
@@ -89,8 +94,8 @@ class TestSeriesAppendWithDatetimeIndex:
         ts = Series(np.random.randn(len(rng)), rng)
         df = DataFrame(np.random.randn(len(rng), 4), index=rng)
 
-        result = ts.append(ts)
-        result_df = df.append(df)
+        result = ts._append(ts)
+        result_df = df._append(df)
         ex_index = DatetimeIndex(np.tile(rng.values, 2))
         tm.assert_index_equal(result.index, ex_index)
         tm.assert_index_equal(result_df.index, ex_index)
@@ -107,6 +112,7 @@ class TestSeriesAppendWithDatetimeIndex:
         rng2 = rng.copy()
         rng1.name = "foo"
         rng2.name = "bar"
+
         assert rng1.append(rng1).name == "foo"
         assert rng1.append(rng2).name is None
 
@@ -120,8 +126,8 @@ class TestSeriesAppendWithDatetimeIndex:
         ts2 = Series(np.random.randn(len(rng2)), rng2)
         df2 = DataFrame(np.random.randn(len(rng2), 4), index=rng2)
 
-        result = ts.append(ts2)
-        result_df = df.append(df2)
+        result = ts._append(ts2)
+        result_df = df._append(df2)
         tm.assert_index_equal(result.index, rng3)
         tm.assert_index_equal(result_df.index, rng3)
 
@@ -146,8 +152,8 @@ class TestSeriesAppendWithDatetimeIndex:
         ts2 = Series(np.random.randn(len(rng2)), rng2)
         df2 = DataFrame(np.random.randn(len(rng2), 4), index=rng2)
 
-        result = ts.append(ts2)
-        result_df = df.append(df2)
+        result = ts._append(ts2)
+        result_df = df._append(df2)
         tm.assert_index_equal(result.index, rng3)
         tm.assert_index_equal(result_df.index, rng3)
 
@@ -170,8 +176,8 @@ class TestSeriesAppendWithDatetimeIndex:
         ts2 = Series(np.random.randn(len(rng2)), rng2)
         df2 = DataFrame(np.random.randn(len(rng2), 4), index=rng2)
 
-        result = ts.append(ts2)
-        result_df = df.append(df2)
+        result = ts._append(ts2)
+        result_df = df._append(df2)
         tm.assert_index_equal(result.index, rng3)
         tm.assert_index_equal(result_df.index, rng3)
 
@@ -183,7 +189,7 @@ class TestSeriesAppendWithDatetimeIndex:
         rng2 = date_range("1/1/2011 02:00", periods=1, freq="H", tz="US/Eastern")
         ser1 = Series([1], index=rng1)
         ser2 = Series([2], index=rng2)
-        ts_result = ser1.append(ser2)
+        ts_result = ser1._append(ser2)
 
         exp_index = DatetimeIndex(
             ["2011-01-01 01:00", "2011-01-01 02:00"], tz="US/Eastern", freq="H"
@@ -196,7 +202,7 @@ class TestSeriesAppendWithDatetimeIndex:
         rng2 = date_range("1/1/2011 02:00", periods=1, freq="H", tz="UTC")
         ser1 = Series([1], index=rng1)
         ser2 = Series([2], index=rng2)
-        ts_result = ser1.append(ser2)
+        ts_result = ser1._append(ser2)
 
         exp_index = DatetimeIndex(
             ["2011-01-01 01:00", "2011-01-01 02:00"], tz="UTC", freq="H"
@@ -212,7 +218,7 @@ class TestSeriesAppendWithDatetimeIndex:
         rng2 = date_range("1/1/2011 02:00", periods=1, freq="H", tz="US/Central")
         ser1 = Series([1], index=rng1)
         ser2 = Series([2], index=rng2)
-        ts_result = ser1.append(ser2)
+        ts_result = ser1._append(ser2)
         exp_index = Index(
             [
                 Timestamp("1/1/2011 01:00", tz="US/Eastern"),
@@ -227,7 +233,7 @@ class TestSeriesAppendWithDatetimeIndex:
         rng2 = date_range("1/1/2011 02:00", periods=1, freq="H", tz="US/Eastern")
         ser1 = Series(np.random.randn(len(rng1)), index=rng1)
         ser2 = Series(np.random.randn(len(rng2)), index=rng2)
-        ts_result = ser1.append(ser2)
+        ts_result = ser1._append(ser2)
 
         expected = ser1.index.astype(object).append(ser2.index.astype(object))
         assert ts_result.index.equals(expected)
@@ -237,7 +243,7 @@ class TestSeriesAppendWithDatetimeIndex:
         rng2 = range(100)
         ser1 = Series(np.random.randn(len(rng1)), index=rng1)
         ser2 = Series(np.random.randn(len(rng2)), index=rng2)
-        ts_result = ser1.append(ser2)
+        ts_result = ser1._append(ser2)
 
         expected = ser1.index.astype(object).append(ser2.index)
         assert ts_result.index.equals(expected)
@@ -247,7 +253,7 @@ class TestSeriesAppendWithDatetimeIndex:
         rng2 = date_range("8/1/2016 01:00", periods=3, freq="H", tz="US/Eastern")
         ser1 = Series([1, 2, 3], index=rng1)
         ser2 = Series([10, 11, 12], index=rng2)
-        ts_result = ser1.append(ser2)
+        ts_result = ser1._append(ser2)
 
         exp_index = DatetimeIndex(
             [
