@@ -9,7 +9,8 @@ from pandas._libs import (
     algos as libalgos,
     hashtable as ht,
 )
-from pandas.compat import np_array_datetime64_compat
+from pandas.compat import pa_version_under7p0
+from pandas.errors import PerformanceWarning
 import pandas.util._test_decorators as td
 
 from pandas.core.dtypes.common import (
@@ -51,15 +52,31 @@ class TestFactorize:
     @pytest.mark.parametrize("sort", [True, False])
     def test_factorize(self, index_or_series_obj, sort):
         obj = index_or_series_obj
-        result_codes, result_uniques = obj.factorize(sort=sort)
+        with tm.maybe_produces_warning(
+            PerformanceWarning,
+            sort
+            and pa_version_under7p0
+            and getattr(obj.dtype, "storage", "") == "pyarrow",
+        ):
+            result_codes, result_uniques = obj.factorize(sort=sort)
 
         constructor = Index
         if isinstance(obj, MultiIndex):
             constructor = MultiIndex.from_tuples
         expected_uniques = constructor(obj.unique())
+        if (
+            isinstance(obj, Index)
+            and expected_uniques.dtype == bool
+            and obj.dtype == object
+        ):
+            expected_uniques = expected_uniques.astype(object)
 
         if sort:
-            expected_uniques = expected_uniques.sort_values()
+            with tm.maybe_produces_warning(
+                PerformanceWarning,
+                pa_version_under7p0 and getattr(obj.dtype, "storage", "") == "pyarrow",
+            ):
+                expected_uniques = expected_uniques.sort_values()
 
         # construct an integer ndarray so that
         # `expected_uniques.take(expected_codes)` is equal to `obj`
@@ -68,13 +85,13 @@ class TestFactorize:
         expected_codes = np.asarray(expected_codes, dtype=np.intp)
 
         tm.assert_numpy_array_equal(result_codes, expected_codes)
-        tm.assert_index_equal(result_uniques, expected_uniques)
+        tm.assert_index_equal(result_uniques, expected_uniques, exact=True)
 
-    def test_series_factorize_na_sentinel_none(self):
+    def test_series_factorize_use_na_sentinel_false(self):
         # GH#35667
         values = np.array([1, 2, 1, np.nan])
         ser = Series(values)
-        codes, uniques = ser.factorize(na_sentinel=None)
+        codes, uniques = ser.factorize(use_na_sentinel=False)
 
         expected_codes = np.array([0, 1, 0, 2], dtype=np.intp)
         expected_uniques = Index([1.0, 2.0, np.nan])
@@ -82,6 +99,20 @@ class TestFactorize:
         tm.assert_numpy_array_equal(codes, expected_codes)
         tm.assert_index_equal(uniques, expected_uniques)
 
+    @pytest.mark.parametrize("na_sentinel", [None, -1, -10])
+    def test_depr_na_sentinel(self, na_sentinel, index_or_series_obj):
+        # GH#46910
+        if na_sentinel is None:
+            msg = "Specifying `na_sentinel=None` is deprecated"
+        elif na_sentinel == -1:
+            msg = "Specifying `na_sentinel=-1` is deprecated"
+        else:
+            msg = "Specifying the specific value to use for `na_sentinel` is deprecated"
+        with tm.assert_produces_warning(FutureWarning, match=msg):
+            pd.factorize(index_or_series_obj, na_sentinel=na_sentinel)
+        with tm.assert_produces_warning(FutureWarning, match=msg):
+            index_or_series_obj.factorize(na_sentinel=na_sentinel)
+
     def test_basic(self):
 
         codes, uniques = algos.factorize(["a", "b", "b", "a", "a", "c", "c", "c"])
@@ -241,17 +272,13 @@ class TestFactorize:
         # gh 12666 - check no segfault
         x17 = np.array([complex(i) for i in range(17)], dtype=object)
 
-        msg = (
-            "unorderable types: .* [<>] .*"
-            "|"  # the above case happens for numpy < 1.14
-            "'[<>]' not supported between instances of .*"
-        )
+        msg = "'[<>]' not supported between instances of .*"
         with pytest.raises(TypeError, match=msg):
             algos.factorize(x17[::-1], sort=True)
 
-    def test_numeric_dtype_factorize(self, any_real_dtype):
+    def test_numeric_dtype_factorize(self, any_real_numpy_dtype):
         # GH41132
-        dtype = any_real_dtype
+        dtype = any_real_numpy_dtype
         data = np.array([1, 2, 2, 1], dtype=dtype)
         expected_codes = np.array([0, 1, 1, 0], dtype=np.intp)
         expected_uniques = np.array([1, 2], dtype=dtype)
@@ -271,20 +298,20 @@ class TestFactorize:
         tm.assert_numpy_array_equal(uniques, expected_uniques)
 
     def test_uint64_factorize(self, writable):
-        data = np.array([2 ** 64 - 1, 1, 2 ** 64 - 1], dtype=np.uint64)
+        data = np.array([2**64 - 1, 1, 2**64 - 1], dtype=np.uint64)
         data.setflags(write=writable)
         expected_codes = np.array([0, 1, 0], dtype=np.intp)
-        expected_uniques = np.array([2 ** 64 - 1, 1], dtype=np.uint64)
+        expected_uniques = np.array([2**64 - 1, 1], dtype=np.uint64)
 
         codes, uniques = algos.factorize(data)
         tm.assert_numpy_array_equal(codes, expected_codes)
         tm.assert_numpy_array_equal(uniques, expected_uniques)
 
     def test_int64_factorize(self, writable):
-        data = np.array([2 ** 63 - 1, -(2 ** 63), 2 ** 63 - 1], dtype=np.int64)
+        data = np.array([2**63 - 1, -(2**63), 2**63 - 1], dtype=np.int64)
         data.setflags(write=writable)
         expected_codes = np.array([0, 1, 0], dtype=np.intp)
-        expected_uniques = np.array([2 ** 63 - 1, -(2 ** 63)], dtype=np.int64)
+        expected_uniques = np.array([2**63 - 1, -(2**63)], dtype=np.int64)
 
         codes, uniques = algos.factorize(data)
         tm.assert_numpy_array_equal(codes, expected_codes)
@@ -359,7 +386,7 @@ class TestFactorize:
     def test_deprecate_order(self):
         # gh 19727 - check warning is raised for deprecated keyword, order.
         # Test not valid once order keyword is removed.
-        data = np.array([2 ** 63, 1, 2 ** 63], dtype=np.uint64)
+        data = np.array([2**63, 1, 2**63], dtype=np.uint64)
         with pytest.raises(TypeError, match="got an unexpected keyword"):
             algos.factorize(data, order=True)
         with tm.assert_produces_warning(False):
@@ -369,7 +396,7 @@ class TestFactorize:
         "data",
         [
             np.array([0, 1, 0], dtype="u8"),
-            np.array([-(2 ** 63), 1, -(2 ** 63)], dtype="i8"),
+            np.array([-(2**63), 1, -(2**63)], dtype="i8"),
             np.array(["__nan__", "foo", "__nan__"], dtype="object"),
         ],
     )
@@ -386,8 +413,8 @@ class TestFactorize:
         [
             (np.array([0, 1, 0, 2], dtype="u8"), 0),
             (np.array([1, 0, 1, 2], dtype="u8"), 1),
-            (np.array([-(2 ** 63), 1, -(2 ** 63), 0], dtype="i8"), -(2 ** 63)),
-            (np.array([1, -(2 ** 63), 1, 0], dtype="i8"), 1),
+            (np.array([-(2**63), 1, -(2**63), 0], dtype="i8"), -(2**63)),
+            (np.array([1, -(2**63), 1, 0], dtype="i8"), 1),
             (np.array(["a", "", "a", "b"], dtype=object), "a"),
             (np.array([(), ("a", 1), (), ("a", 2)], dtype=object), ()),
             (np.array([("a", 1), (), ("a", 1), ("a", 2)], dtype=object), ("a", 1)),
@@ -417,7 +444,12 @@ class TestFactorize:
         ids=["numpy_array", "extension_array"],
     )
     def test_factorize_na_sentinel(self, sort, na_sentinel, data, uniques):
-        codes, uniques = algos.factorize(data, sort=sort, na_sentinel=na_sentinel)
+        if na_sentinel == -1:
+            msg = "Specifying `na_sentinel=-1` is deprecated"
+        else:
+            msg = "the specific value to use for `na_sentinel` is deprecated"
+        with tm.assert_produces_warning(FutureWarning, match=msg):
+            codes, uniques = algos.factorize(data, sort=sort, na_sentinel=na_sentinel)
         if sort:
             expected_codes = np.array([1, 0, na_sentinel, 1], dtype=np.intp)
             expected_uniques = algos.safe_sort(uniques)
@@ -435,46 +467,46 @@ class TestFactorize:
         [
             (
                 ["a", None, "b", "a"],
-                np.array([0, 2, 1, 0], dtype=np.dtype("intp")),
-                np.array(["a", "b", np.nan], dtype=object),
+                np.array([0, 1, 2, 0], dtype=np.dtype("intp")),
+                np.array(["a", np.nan, "b"], dtype=object),
             ),
             (
                 ["a", np.nan, "b", "a"],
-                np.array([0, 2, 1, 0], dtype=np.dtype("intp")),
-                np.array(["a", "b", np.nan], dtype=object),
+                np.array([0, 1, 2, 0], dtype=np.dtype("intp")),
+                np.array(["a", np.nan, "b"], dtype=object),
             ),
         ],
     )
-    def test_object_factorize_na_sentinel_none(
+    def test_object_factorize_use_na_sentinel_false(
         self, data, expected_codes, expected_uniques
     ):
-        codes, uniques = algos.factorize(data, na_sentinel=None)
+        codes, uniques = algos.factorize(data, use_na_sentinel=False)
 
-        tm.assert_numpy_array_equal(uniques, expected_uniques)
-        tm.assert_numpy_array_equal(codes, expected_codes)
+        tm.assert_numpy_array_equal(uniques, expected_uniques, strict_nan=True)
+        tm.assert_numpy_array_equal(codes, expected_codes, strict_nan=True)
 
     @pytest.mark.parametrize(
         "data, expected_codes, expected_uniques",
         [
             (
                 [1, None, 1, 2],
-                np.array([0, 2, 0, 1], dtype=np.dtype("intp")),
-                np.array([1, 2, np.nan], dtype="O"),
+                np.array([0, 1, 0, 2], dtype=np.dtype("intp")),
+                np.array([1, np.nan, 2], dtype="O"),
             ),
             (
                 [1, np.nan, 1, 2],
-                np.array([0, 2, 0, 1], dtype=np.dtype("intp")),
-                np.array([1, 2, np.nan], dtype=np.float64),
+                np.array([0, 1, 0, 2], dtype=np.dtype("intp")),
+                np.array([1, np.nan, 2], dtype=np.float64),
             ),
         ],
     )
-    def test_int_factorize_na_sentinel_none(
+    def test_int_factorize_use_na_sentinel_false(
         self, data, expected_codes, expected_uniques
     ):
-        codes, uniques = algos.factorize(data, na_sentinel=None)
+        codes, uniques = algos.factorize(data, use_na_sentinel=False)
 
-        tm.assert_numpy_array_equal(uniques, expected_uniques)
-        tm.assert_numpy_array_equal(codes, expected_codes)
+        tm.assert_numpy_array_equal(uniques, expected_uniques, strict_nan=True)
+        tm.assert_numpy_array_equal(codes, expected_codes, strict_nan=True)
 
 
 class TestUnique:
@@ -513,7 +545,8 @@ class TestUnique:
     def test_dtype_preservation(self, any_numpy_dtype):
         # GH 15442
         if any_numpy_dtype in (tm.BYTES_DTYPES + tm.STRING_DTYPES):
-            pytest.skip("skip string dtype")
+            data = [1, 2, 2]
+            uniques = [1, 2]
         elif is_integer_dtype(any_numpy_dtype):
             data = [1, 2, 2]
             uniques = [1, 2]
@@ -537,14 +570,17 @@ class TestUnique:
         result = Series(data, dtype=any_numpy_dtype).unique()
         expected = np.array(uniques, dtype=any_numpy_dtype)
 
+        if any_numpy_dtype in tm.STRING_DTYPES:
+            expected = expected.astype(object)
+
         tm.assert_numpy_array_equal(result, expected)
 
     def test_datetime64_dtype_array_returned(self):
         # GH 9431
-        expected = np_array_datetime64_compat(
+        expected = np.array(
             [
-                "2015-01-03T00:00:00.000000000+0000",
-                "2015-01-01T00:00:00.000000000+0000",
+                "2015-01-03T00:00:00.000000000",
+                "2015-01-01T00:00:00.000000000",
             ],
             dtype="M8[ns]",
         )
@@ -602,8 +638,8 @@ class TestUnique:
         assert result.dtype == expected.dtype
 
     def test_uint64_overflow(self):
-        s = Series([1, 2, 2 ** 63, 2 ** 63], dtype=np.uint64)
-        exp = np.array([1, 2, 2 ** 63], dtype=np.uint64)
+        s = Series([1, 2, 2**63, 2**63], dtype=np.uint64)
+        exp = np.array([1, 2, 2**63], dtype=np.uint64)
         tm.assert_numpy_array_equal(algos.unique(s), exp)
 
     def test_nan_in_object_array(self):
@@ -783,7 +819,8 @@ class TestUnique:
         expected = np.array([np.nan])
         tm.assert_numpy_array_equal(result, expected)
 
-    def test_first_nan_kept(self):
+    @pytest.mark.parametrize("el_type", [np.float64, object])
+    def test_first_nan_kept(self, el_type):
         # GH 22295
         # create different nans from bit-patterns:
         bits_for_nan1 = 0xFFF8000000000001
@@ -792,13 +829,12 @@ class TestUnique:
         NAN2 = struct.unpack("d", struct.pack("=Q", bits_for_nan2))[0]
         assert NAN1 != NAN1
         assert NAN2 != NAN2
-        for el_type in [np.float64, object]:
-            a = np.array([NAN1, NAN2], dtype=el_type)
-            result = pd.unique(a)
-            assert result.size == 1
-            # use bit patterns to identify which nan was kept:
-            result_nan_bits = struct.unpack("=Q", struct.pack("d", result[0]))[0]
-            assert result_nan_bits == bits_for_nan1
+        a = np.array([NAN1, NAN2], dtype=el_type)
+        result = pd.unique(a)
+        assert result.size == 1
+        # use bit patterns to identify which nan was kept:
+        result_nan_bits = struct.unpack("=Q", struct.pack("d", result[0]))[0]
+        assert result_nan_bits == bits_for_nan1
 
     def test_do_not_mangle_na_values(self, unique_nulls_fixture, unique_nulls_fixture2):
         # GH 22295
@@ -810,6 +846,13 @@ class TestUnique:
         assert a[0] is unique_nulls_fixture
         assert a[1] is unique_nulls_fixture2
 
+    def test_unique_masked(self, any_numeric_ea_dtype):
+        # GH#48019
+        ser = Series([1, pd.NA, 2] * 3, dtype=any_numeric_ea_dtype)
+        result = pd.unique(ser)
+        expected = pd.array([1, pd.NA, 2], dtype=any_numeric_ea_dtype)
+        tm.assert_extension_array_equal(result, expected)
+
 
 class TestIsin:
     def test_invalid(self):
@@ -1084,6 +1127,13 @@ class TestIsin:
         expected_false = DataFrame({"values": [False, False]})
         tm.assert_frame_equal(result, expected_false)
 
+    def test_isin_unsigned_dtype(self):
+        # GH#46485
+        ser = Series([1378774140726870442], dtype=np.uint64)
+        result = ser.isin([1378774140726870528])
+        expected = Series(False)
+        tm.assert_series_equal(result, expected)
+
 
 class TestValueCounts:
     def test_value_counts(self):
@@ -1241,7 +1291,7 @@ class TestValueCounts:
 
         tm.assert_series_equal(
             Series([True] * 3 + [False] * 2 + [None] * 5).value_counts(dropna=True),
-            Series([3, 2], index=[True, False]),
+            Series([3, 2], index=Index([True, False], dtype=object)),
         )
         tm.assert_series_equal(
             Series([True] * 5 + [False] * 3 + [None] * 2).value_counts(dropna=False),
@@ -1265,31 +1315,30 @@ class TestValueCounts:
         expected = Series([3, 2, 1], index=[5.0, 10.3, np.nan])
         tm.assert_series_equal(result, expected)
 
-    def test_value_counts_normalized(self):
+    @pytest.mark.parametrize("dtype", (np.float64, object, "M8[ns]"))
+    def test_value_counts_normalized(self, dtype):
         # GH12558
         s = Series([1] * 2 + [2] * 3 + [np.nan] * 5)
-        dtypes = (np.float64, object, "M8[ns]")
-        for t in dtypes:
-            s_typed = s.astype(t)
-            result = s_typed.value_counts(normalize=True, dropna=False)
-            expected = Series(
-                [0.5, 0.3, 0.2], index=Series([np.nan, 2.0, 1.0], dtype=t)
-            )
-            tm.assert_series_equal(result, expected)
+        s_typed = s.astype(dtype)
+        result = s_typed.value_counts(normalize=True, dropna=False)
+        expected = Series(
+            [0.5, 0.3, 0.2], index=Series([np.nan, 2.0, 1.0], dtype=dtype)
+        )
+        tm.assert_series_equal(result, expected)
 
-            result = s_typed.value_counts(normalize=True, dropna=True)
-            expected = Series([0.6, 0.4], index=Series([2.0, 1.0], dtype=t))
-            tm.assert_series_equal(result, expected)
+        result = s_typed.value_counts(normalize=True, dropna=True)
+        expected = Series([0.6, 0.4], index=Series([2.0, 1.0], dtype=dtype))
+        tm.assert_series_equal(result, expected)
 
     def test_value_counts_uint64(self):
-        arr = np.array([2 ** 63], dtype=np.uint64)
-        expected = Series([1], index=[2 ** 63])
+        arr = np.array([2**63], dtype=np.uint64)
+        expected = Series([1], index=[2**63])
         result = algos.value_counts(arr)
 
         tm.assert_series_equal(result, expected)
 
-        arr = np.array([-1, 2 ** 63], dtype=object)
-        expected = Series([1, 1], index=[-1, 2 ** 63])
+        arr = np.array([-1, 2**63], dtype=object)
+        expected = Series([1, 1], index=[-1, 2**63])
         result = algos.value_counts(arr)
 
         tm.assert_series_equal(result, expected)
@@ -1356,7 +1405,7 @@ class TestDuplicated:
             ),
             np.array(["a", "b", "a", "e", "c", "b", "d", "a", "e", "f"], dtype=object),
             np.array(
-                [1, 2 ** 63, 1, 3 ** 5, 10, 2 ** 63, 39, 1, 3 ** 5, 7], dtype=np.uint64
+                [1, 2**63, 1, 3**5, 10, 2**63, 39, 1, 3**5, 7], dtype=np.uint64
             ),
         ],
     )
@@ -1483,13 +1532,10 @@ class TestDuplicated:
                 res_false = s.duplicated(keep=False)
                 tm.assert_series_equal(res_false, Series(exp_false))
 
-    def test_unique_index(self):
-        cases = [Index([1, 2, 3]), pd.RangeIndex(0, 3)]
-        for case in cases:
-            assert case.is_unique is True
-            tm.assert_numpy_array_equal(
-                case.duplicated(), np.array([False, False, False])
-            )
+    @pytest.mark.parametrize("case", [Index([1, 2, 3]), pd.RangeIndex(0, 3)])
+    def test_unique_index(self, case):
+        assert case.is_unique is True
+        tm.assert_numpy_array_equal(case.duplicated(), np.array([False, False, False]))
 
     @pytest.mark.parametrize(
         "arr, uniques",
@@ -1513,112 +1559,23 @@ class TestDuplicated:
         result = pd.unique(arr)
         tm.assert_numpy_array_equal(result, expected)
 
-
-class TestHashTable:
-    def test_string_hashtable_set_item_signature(self):
-        # GH#30419 fix typing in StringHashTable.set_item to prevent segfault
-        tbl = ht.StringHashTable()
-
-        tbl.set_item("key", 1)
-        assert tbl.get_item("key") == 1
-
-        with pytest.raises(TypeError, match="'key' has incorrect type"):
-            # key arg typed as string, not object
-            tbl.set_item(4, 6)
-        with pytest.raises(TypeError, match="'val' has incorrect type"):
-            tbl.get_item(4)
-
-    def test_lookup_nan(self, writable):
-        xs = np.array([2.718, 3.14, np.nan, -7, 5, 2, 3])
-        # GH 21688 ensure we can deal with readonly memory views
-        xs.setflags(write=writable)
-        m = ht.Float64HashTable()
-        m.map_locations(xs)
-        tm.assert_numpy_array_equal(m.lookup(xs), np.arange(len(xs), dtype=np.intp))
-
-    def test_add_signed_zeros(self):
-        # GH 21866 inconsistent hash-function for float64
-        # default hash-function would lead to different hash-buckets
-        # for 0.0 and -0.0 if there are more than 2^30 hash-buckets
-        # but this would mean 16GB
-        N = 4  # 12 * 10**8 would trigger the error, if you have enough memory
-        m = ht.Float64HashTable(N)
-        m.set_item(0.0, 0)
-        m.set_item(-0.0, 0)
-        assert len(m) == 1  # 0.0 and -0.0 are equivalent
-
-    def test_add_different_nans(self):
-        # GH 21866 inconsistent hash-function for float64
-        # create different nans from bit-patterns:
-        NAN1 = struct.unpack("d", struct.pack("=Q", 0x7FF8000000000000))[0]
-        NAN2 = struct.unpack("d", struct.pack("=Q", 0x7FF8000000000001))[0]
-        assert NAN1 != NAN1
-        assert NAN2 != NAN2
-        # default hash function would lead to different hash-buckets
-        # for NAN1 and NAN2 even if there are only 4 buckets:
-        m = ht.Float64HashTable()
-        m.set_item(NAN1, 0)
-        m.set_item(NAN2, 0)
-        assert len(m) == 1  # NAN1 and NAN2 are equivalent
-
-    def test_lookup_overflow(self, writable):
-        xs = np.array([1, 2, 2 ** 63], dtype=np.uint64)
-        # GH 21688 ensure we can deal with readonly memory views
-        xs.setflags(write=writable)
-        m = ht.UInt64HashTable()
-        m.map_locations(xs)
-        tm.assert_numpy_array_equal(m.lookup(xs), np.arange(len(xs), dtype=np.intp))
-
-    def test_get_unique(self):
-        s = Series([1, 2, 2 ** 63, 2 ** 63], dtype=np.uint64)
-        exp = np.array([1, 2, 2 ** 63], dtype=np.uint64)
-        tm.assert_numpy_array_equal(s.unique(), exp)
-
-    @pytest.mark.parametrize("nvals", [0, 10])  # resizing to 0 is special case
     @pytest.mark.parametrize(
-        "htable, uniques, dtype, safely_resizes",
+        "array,expected",
         [
-            (ht.PyObjectHashTable, ht.ObjectVector, "object", False),
-            (ht.StringHashTable, ht.ObjectVector, "object", True),
-            (ht.Float64HashTable, ht.Float64Vector, "float64", False),
-            (ht.Int64HashTable, ht.Int64Vector, "int64", False),
-            (ht.Int32HashTable, ht.Int32Vector, "int32", False),
-            (ht.UInt64HashTable, ht.UInt64Vector, "uint64", False),
+            (
+                [1 + 1j, 0, 1, 1j, 1 + 2j, 1 + 2j],
+                # Should return a complex dtype in the future
+                np.array([(1 + 1j), 0j, (1 + 0j), 1j, (1 + 2j)], dtype=object),
+            )
         ],
     )
-    def test_vector_resize(
-        self, writable, htable, uniques, dtype, safely_resizes, nvals
-    ):
-        # Test for memory errors after internal vector
-        # reallocations (GH 7157)
-        vals = np.array(np.random.randn(1000), dtype=dtype)
+    def test_unique_complex_numbers(self, array, expected):
+        # GH 17927
+        result = pd.unique(array)
+        tm.assert_numpy_array_equal(result, expected)
 
-        # GH 21688 ensures we can deal with read-only memory views
-        vals.setflags(write=writable)
-
-        # initialise instances; cannot initialise in parametrization,
-        # as otherwise external views would be held on the array (which is
-        # one of the things this test is checking)
-        htable = htable()
-        uniques = uniques()
-
-        # get_labels may append to uniques
-        htable.get_labels(vals[:nvals], uniques, 0, -1)
-        # to_array() sets an external_view_exists flag on uniques.
-        tmp = uniques.to_array()
-        oldshape = tmp.shape
-
-        # subsequent get_labels() calls can no longer append to it
-        # (except for StringHashTables + ObjectVector)
-        if safely_resizes:
-            htable.get_labels(vals, uniques, 0, -1)
-        else:
-            with pytest.raises(ValueError, match="external reference.*"):
-                htable.get_labels(vals, uniques, 0, -1)
-
-        uniques.to_array()  # should not raise here
-        assert tmp.shape == oldshape
 
+class TestHashTable:
     @pytest.mark.parametrize(
         "htable, tm_dtype",
         [
@@ -1699,62 +1656,28 @@ class TestHashTable:
         expected_reconstruct = s_duplicated.dropna().values
         tm.assert_numpy_array_equal(result_reconstruct, expected_reconstruct)
 
-    @pytest.mark.parametrize(
-        "hashtable",
-        [
-            ht.PyObjectHashTable,
-            ht.StringHashTable,
-            ht.Float64HashTable,
-            ht.Int64HashTable,
-            ht.Int32HashTable,
-            ht.UInt64HashTable,
-        ],
-    )
-    def test_hashtable_large_sizehint(self, hashtable):
-        # GH 22729
-        size_hint = np.iinfo(np.uint32).max + 1
-        tbl = hashtable(size_hint=size_hint)  # noqa
-
-
-def test_quantile():
-    s = Series(np.random.randn(100))
-
-    result = algos.quantile(s, [0, 0.25, 0.5, 0.75, 1.0])
-    expected = algos.quantile(s.values, [0, 0.25, 0.5, 0.75, 1.0])
-    tm.assert_almost_equal(result, expected)
-
-
-def test_unique_label_indices():
-
-    a = np.random.randint(1, 1 << 10, 1 << 15).astype("int64")
-
-    left = ht.unique_label_indices(a)
-    right = np.unique(a, return_index=True)[1]
-
-    tm.assert_numpy_array_equal(left, right, check_dtype=False)
-
-    a[np.random.choice(len(a), 10)] = -1
-    left = ht.unique_label_indices(a)
-    right = np.unique(a, return_index=True)[1][1:]
-    tm.assert_numpy_array_equal(left, right, check_dtype=False)
-
 
 class TestRank:
     @td.skip_if_no_scipy
-    def test_scipy_compat(self):
+    @pytest.mark.parametrize(
+        "arr",
+        [
+            [np.nan, np.nan, 5.0, 5.0, 5.0, np.nan, 1, 2, 3, np.nan],
+            [4.0, np.nan, 5.0, 5.0, 5.0, np.nan, 1, 2, 4.0, np.nan],
+        ],
+    )
+    def test_scipy_compat(self, arr):
         from scipy.stats import rankdata
 
-        def _check(arr):
-            mask = ~np.isfinite(arr)
-            arr = arr.copy()
-            result = libalgos.rank_1d(arr, labels=np.zeros(len(arr), dtype=np.intp))
-            arr[mask] = np.inf
-            exp = rankdata(arr)
-            exp[mask] = np.nan
-            tm.assert_almost_equal(result, exp)
+        arr = np.array(arr)
 
-        _check(np.array([np.nan, np.nan, 5.0, 5.0, 5.0, np.nan, 1, 2, 3, np.nan]))
-        _check(np.array([4.0, np.nan, 5.0, 5.0, 5.0, np.nan, 1, 2, 4.0, np.nan]))
+        mask = ~np.isfinite(arr)
+        arr = arr.copy()
+        result = libalgos.rank_1d(arr)
+        arr[mask] = np.inf
+        exp = rankdata(arr)
+        exp[mask] = np.nan
+        tm.assert_almost_equal(result, exp)
 
     @pytest.mark.parametrize("dtype", np.typecodes["AllInteger"])
     def test_basic(self, writable, dtype):
@@ -1766,12 +1689,12 @@ class TestRank:
         result = algos.rank(ser)
         tm.assert_numpy_array_equal(result, exp)
 
-    def test_uint64_overflow(self):
+    @pytest.mark.parametrize("dtype", [np.float64, np.uint64])
+    def test_uint64_overflow(self, dtype):
         exp = np.array([1, 2], dtype=np.float64)
 
-        for dtype in [np.float64, np.uint64]:
-            s = Series([1, 2 ** 63], dtype=dtype)
-            tm.assert_numpy_array_equal(algos.rank(s), exp)
+        s = Series([1, 2**63], dtype=dtype)
+        tm.assert_numpy_array_equal(algos.rank(s), exp)
 
     def test_too_many_ndims(self):
         arr = np.array([[[1, 2, 3], [4, 5, 6], [7, 8, 9]]])
@@ -1780,15 +1703,14 @@ class TestRank:
         with pytest.raises(TypeError, match=msg):
             algos.rank(arr)
 
-    @pytest.mark.single
-    @pytest.mark.high_memory
-    @pytest.mark.parametrize(
-        "values",
-        [np.arange(2 ** 24 + 1), np.arange(2 ** 25 + 2).reshape(2 ** 24 + 1, 2)],
-        ids=["1d", "2d"],
-    )
-    def test_pct_max_many_rows(self, values):
+    @pytest.mark.single_cpu
+    def test_pct_max_many_rows(self):
         # GH 18271
+        values = np.arange(2**24 + 1)
+        result = algos.rank(values, pct=True).max()
+        assert result == 1
+
+        values = np.arange(2**25 + 2).reshape(2**24 + 1, 2)
         result = algos.rank(values, pct=True).max()
         assert result == 1
 
@@ -1816,21 +1738,6 @@ def test_pad_backfill_object_segfault():
 
 
 class TestTseriesUtil:
-    def test_combineFunc(self):
-        pass
-
-    def test_reindex(self):
-        pass
-
-    def test_isna(self):
-        pass
-
-    def test_groupby(self):
-        pass
-
-    def test_groupby_withnull(self):
-        pass
-
     def test_backfill(self):
         old = Index([1, 5, 10])
         new = Index(list(range(12)))
@@ -2269,9 +2176,10 @@ def test_int64_add_overflow():
 class TestMode:
     def test_no_mode(self):
         exp = Series([], dtype=np.float64, index=Index([], dtype=int))
-        tm.assert_series_equal(algos.mode([]), exp)
+        tm.assert_numpy_array_equal(algos.mode([]), exp.values)
 
-    def test_mode_single(self):
+    @pytest.mark.parametrize("dt", np.typecodes["AllInteger"] + np.typecodes["Float"])
+    def test_mode_single(self, dt):
         # GH 15714
         exp_single = [1]
         data_single = [1]
@@ -2279,126 +2187,152 @@ class TestMode:
         exp_multi = [1]
         data_multi = [1, 1]
 
-        for dt in np.typecodes["AllInteger"] + np.typecodes["Float"]:
-            s = Series(data_single, dtype=dt)
-            exp = Series(exp_single, dtype=dt)
-            tm.assert_series_equal(algos.mode(s), exp)
+        ser = Series(data_single, dtype=dt)
+        exp = Series(exp_single, dtype=dt)
+        tm.assert_numpy_array_equal(algos.mode(ser.values), exp.values)
+        tm.assert_series_equal(ser.mode(), exp)
 
-            s = Series(data_multi, dtype=dt)
-            exp = Series(exp_multi, dtype=dt)
-            tm.assert_series_equal(algos.mode(s), exp)
+        ser = Series(data_multi, dtype=dt)
+        exp = Series(exp_multi, dtype=dt)
+        tm.assert_numpy_array_equal(algos.mode(ser.values), exp.values)
+        tm.assert_series_equal(ser.mode(), exp)
 
+    def test_mode_obj_int(self):
         exp = Series([1], dtype=int)
-        tm.assert_series_equal(algos.mode([1]), exp)
+        tm.assert_numpy_array_equal(algos.mode([1]), exp.values)
 
         exp = Series(["a", "b", "c"], dtype=object)
-        tm.assert_series_equal(algos.mode(["a", "b", "c"]), exp)
+        tm.assert_numpy_array_equal(algos.mode(["a", "b", "c"]), exp.values)
 
-    def test_number_mode(self):
+    @pytest.mark.parametrize("dt", np.typecodes["AllInteger"] + np.typecodes["Float"])
+    def test_number_mode(self, dt):
         exp_single = [1]
         data_single = [1] * 5 + [2] * 3
 
         exp_multi = [1, 3]
         data_multi = [1] * 5 + [2] * 3 + [3] * 5
 
-        for dt in np.typecodes["AllInteger"] + np.typecodes["Float"]:
-            s = Series(data_single, dtype=dt)
-            exp = Series(exp_single, dtype=dt)
-            tm.assert_series_equal(algos.mode(s), exp)
+        ser = Series(data_single, dtype=dt)
+        exp = Series(exp_single, dtype=dt)
+        tm.assert_numpy_array_equal(algos.mode(ser.values), exp.values)
+        tm.assert_series_equal(ser.mode(), exp)
 
-            s = Series(data_multi, dtype=dt)
-            exp = Series(exp_multi, dtype=dt)
-            tm.assert_series_equal(algos.mode(s), exp)
+        ser = Series(data_multi, dtype=dt)
+        exp = Series(exp_multi, dtype=dt)
+        tm.assert_numpy_array_equal(algos.mode(ser.values), exp.values)
+        tm.assert_series_equal(ser.mode(), exp)
 
     def test_strobj_mode(self):
         exp = ["b"]
         data = ["a"] * 2 + ["b"] * 3
 
-        s = Series(data, dtype="c")
+        ser = Series(data, dtype="c")
         exp = Series(exp, dtype="c")
-        tm.assert_series_equal(algos.mode(s), exp)
+        tm.assert_numpy_array_equal(algos.mode(ser.values), exp.values)
+        tm.assert_series_equal(ser.mode(), exp)
 
+    @pytest.mark.parametrize("dt", [str, object])
+    def test_strobj_multi_char(self, dt):
         exp = ["bar"]
         data = ["foo"] * 2 + ["bar"] * 3
 
-        for dt in [str, object]:
-            s = Series(data, dtype=dt)
-            exp = Series(exp, dtype=dt)
-            tm.assert_series_equal(algos.mode(s), exp)
+        ser = Series(data, dtype=dt)
+        exp = Series(exp, dtype=dt)
+        tm.assert_numpy_array_equal(algos.mode(ser.values), exp.values)
+        tm.assert_series_equal(ser.mode(), exp)
 
     def test_datelike_mode(self):
         exp = Series(["1900-05-03", "2011-01-03", "2013-01-02"], dtype="M8[ns]")
-        s = Series(["2011-01-03", "2013-01-02", "1900-05-03"], dtype="M8[ns]")
-        tm.assert_series_equal(algos.mode(s), exp)
+        ser = Series(["2011-01-03", "2013-01-02", "1900-05-03"], dtype="M8[ns]")
+        tm.assert_extension_array_equal(algos.mode(ser.values), exp._values)
+        tm.assert_series_equal(ser.mode(), exp)
 
         exp = Series(["2011-01-03", "2013-01-02"], dtype="M8[ns]")
-        s = Series(
+        ser = Series(
             ["2011-01-03", "2013-01-02", "1900-05-03", "2011-01-03", "2013-01-02"],
             dtype="M8[ns]",
         )
-        tm.assert_series_equal(algos.mode(s), exp)
+        tm.assert_extension_array_equal(algos.mode(ser.values), exp._values)
+        tm.assert_series_equal(ser.mode(), exp)
 
     def test_timedelta_mode(self):
         exp = Series(["-1 days", "0 days", "1 days"], dtype="timedelta64[ns]")
-        s = Series(["1 days", "-1 days", "0 days"], dtype="timedelta64[ns]")
-        tm.assert_series_equal(algos.mode(s), exp)
+        ser = Series(["1 days", "-1 days", "0 days"], dtype="timedelta64[ns]")
+        tm.assert_extension_array_equal(algos.mode(ser.values), exp._values)
+        tm.assert_series_equal(ser.mode(), exp)
 
         exp = Series(["2 min", "1 day"], dtype="timedelta64[ns]")
-        s = Series(
+        ser = Series(
             ["1 day", "1 day", "-1 day", "-1 day 2 min", "2 min", "2 min"],
             dtype="timedelta64[ns]",
         )
-        tm.assert_series_equal(algos.mode(s), exp)
+        tm.assert_extension_array_equal(algos.mode(ser.values), exp._values)
+        tm.assert_series_equal(ser.mode(), exp)
 
     def test_mixed_dtype(self):
         exp = Series(["foo"])
-        s = Series([1, "foo", "foo"])
-        tm.assert_series_equal(algos.mode(s), exp)
+        ser = Series([1, "foo", "foo"])
+        tm.assert_numpy_array_equal(algos.mode(ser.values), exp.values)
+        tm.assert_series_equal(ser.mode(), exp)
 
     def test_uint64_overflow(self):
-        exp = Series([2 ** 63], dtype=np.uint64)
-        s = Series([1, 2 ** 63, 2 ** 63], dtype=np.uint64)
-        tm.assert_series_equal(algos.mode(s), exp)
+        exp = Series([2**63], dtype=np.uint64)
+        ser = Series([1, 2**63, 2**63], dtype=np.uint64)
+        tm.assert_numpy_array_equal(algos.mode(ser.values), exp.values)
+        tm.assert_series_equal(ser.mode(), exp)
 
-        exp = Series([1, 2 ** 63], dtype=np.uint64)
-        s = Series([1, 2 ** 63], dtype=np.uint64)
-        tm.assert_series_equal(algos.mode(s), exp)
+        exp = Series([1, 2**63], dtype=np.uint64)
+        ser = Series([1, 2**63], dtype=np.uint64)
+        tm.assert_numpy_array_equal(algos.mode(ser.values), exp.values)
+        tm.assert_series_equal(ser.mode(), exp)
 
     def test_categorical(self):
         c = Categorical([1, 2])
         exp = c
-        tm.assert_categorical_equal(algos.mode(c), exp)
-        tm.assert_categorical_equal(c.mode(), exp)
+        msg = "Categorical.mode is deprecated"
+        with tm.assert_produces_warning(FutureWarning, match=msg):
+            res = c.mode()
+        tm.assert_categorical_equal(res, exp)
 
         c = Categorical([1, "a", "a"])
         exp = Categorical(["a"], categories=[1, "a"])
-        tm.assert_categorical_equal(algos.mode(c), exp)
-        tm.assert_categorical_equal(c.mode(), exp)
+        with tm.assert_produces_warning(FutureWarning, match=msg):
+            res = c.mode()
+        tm.assert_categorical_equal(res, exp)
 
         c = Categorical([1, 1, 2, 3, 3])
         exp = Categorical([1, 3], categories=[1, 2, 3])
-        tm.assert_categorical_equal(algos.mode(c), exp)
-        tm.assert_categorical_equal(c.mode(), exp)
+        with tm.assert_produces_warning(FutureWarning, match=msg):
+            res = c.mode()
+        tm.assert_categorical_equal(res, exp)
 
     def test_index(self):
         idx = Index([1, 2, 3])
         exp = Series([1, 2, 3], dtype=np.int64)
-        tm.assert_series_equal(algos.mode(idx), exp)
+        tm.assert_numpy_array_equal(algos.mode(idx), exp.values)
 
         idx = Index([1, "a", "a"])
         exp = Series(["a"], dtype=object)
-        tm.assert_series_equal(algos.mode(idx), exp)
+        tm.assert_numpy_array_equal(algos.mode(idx), exp.values)
 
         idx = Index([1, 1, 2, 3, 3])
         exp = Series([1, 3], dtype=np.int64)
-        tm.assert_series_equal(algos.mode(idx), exp)
+        tm.assert_numpy_array_equal(algos.mode(idx), exp.values)
 
-        exp = Series(["2 min", "1 day"], dtype="timedelta64[ns]")
         idx = Index(
             ["1 day", "1 day", "-1 day", "-1 day 2 min", "2 min", "2 min"],
             dtype="timedelta64[ns]",
         )
-        tm.assert_series_equal(algos.mode(idx), exp)
+        with pytest.raises(AttributeError, match="TimedeltaIndex"):
+            # algos.mode expects Arraylike, does *not* unwrap TimedeltaIndex
+            algos.mode(idx)
+
+    def test_ser_mode_with_name(self):
+        # GH 46737
+        ser = Series([1, 1, 3], name="foo")
+        result = ser.mode()
+        expected = Series([1], name="foo")
+        tm.assert_series_equal(result, expected)
 
 
 class TestDiff:
@@ -2443,6 +2377,5 @@ def test_union_with_duplicates(op):
         result = algos.union_with_duplicates(lvals, rvals)
         tm.assert_numpy_array_equal(result, expected)
     else:
-        with tm.assert_produces_warning(RuntimeWarning):
-            result = algos.union_with_duplicates(lvals, rvals)
+        result = algos.union_with_duplicates(lvals, rvals)
         tm.assert_extension_array_equal(result, expected)
