@@ -1,6 +1,8 @@
 import warnings
 
-from pandas._testing import *  # noqa
+from pandas.util._exceptions import find_stack_level
+
+from pandas._testing import *  # noqa:F401,F403,PDF014
 
 warnings.warn(
     (
@@ -8,5 +10,5 @@ warnings.warn(
         "public API at pandas.testing instead."
     ),
     FutureWarning,
-    stacklevel=2,
+    stacklevel=find_stack_level(),
 )
